import math
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy import integrate  # optimize, stats
from dataclasses import dataclass, field
from ..common.math.numerical import xy_with_endpoints
from ..common.io.output import OutputTable, OutputExtended
from ..common.io.lists_and_dicts_and_pandas_etc import convert_dict_to_quantity_value
from cached_property import cached_property # https://pypi.org/project/cached-property/


def NigamJennings(acc, periods, damping, dt):
    add_PGA = False
    if periods[0] == 0:
        periods = np.delete(periods, 0)
        add_PGA = True

    num_steps = len(acc)
    num_per = len(periods)
    omega = (2. * np.pi) / np.array(periods)
    omega2 = omega ** 2.
    omega3 = omega ** 3.
    omega_d = omega * math.sqrt(1.0 - (damping ** 2.))
    const = {'f1': (2.0 * damping) / (omega3 * dt),
             'f2': 1.0 / omega2,
             'f3': damping * omega,
             'f4': 1.0 / omega_d}
    const['f5'] = const['f3'] * const['f4']
    const['f6'] = 2.0 * const['f3']
    const['e'] = np.exp(-const['f3'] * dt)
    const['s'] = np.sin(omega_d * dt)
    const['c'] = np.cos(omega_d * dt)
    const['g1'] = const['e'] * const['s']
    const['g2'] = const['e'] * const['c']
    const['h1'] = (omega_d * const['g2']) - (const['f3'] * const['g1'])
    const['h2'] = (omega_d * const['g1']) + (const['f3'] * const['g2'])
    x_a, x_v, x_d = calculate_time_series(num_steps, num_per, acc, const, omega2, dt)

    spectrum = {'Sa': None,
                'Sv': None,
                'Sd': None,
                'T': None,
                'PSv': None,
                'PSa': None,
                'ASI': None,
                'VSI': None,
                'HI': None,
                'Tp': None}

    spectrum['Sa'] = np.max(np.fabs(x_a), axis=0)
    spectrum['Sv'] = np.max(np.fabs(x_v), axis=0)
    spectrum['Sd'] = np.max(np.fabs(x_d), axis=0)
    spectrum['PSv'] = spectrum['Sd'] * omega
    spectrum['PSa'] = spectrum['Sd'] * omega2
    spectrum['T'] = periods

    if add_PGA:
        pga = np.max(np.fabs(acc))
        periods = np.append(0, periods)
        spectrum['Sa'] = np.append(pga, spectrum['Sa'])
        spectrum['Sv'] = np.append(0, spectrum['Sv'])
        spectrum['Sd'] = np.append(0, spectrum['Sd'])
        spectrum['PSv'] = np.append(0, spectrum['PSv'])
        spectrum['PSa'] = np.append(pga, spectrum['PSa'])
        spectrum['T'] = periods

    xy_ASI = xy_with_endpoints(spectrum['T'], spectrum['Sa'], 0.1, 0.5)
    spectrum['ASI'] = integrate.trapz(xy_ASI[1], xy_ASI[0])

    xy_VSI = xy_with_endpoints(spectrum['T'], spectrum['Sv'], 0.1, 2.5)
    spectrum['VSI'] = integrate.trapz(xy_VSI[1], xy_VSI[0])

    xy_HI = xy_with_endpoints(spectrum['T'], spectrum['PSv'], 0.1, 2.5)
    spectrum['HI'] = integrate.trapz(xy_HI[1], xy_HI[0])

    spectrum['Tp'] = (float(np.where(spectrum['Sa'] == max(spectrum['Sa']))[0]) + 1) * (periods[1] - periods[0])

    return spectrum


def calculate_time_series(num_steps, num_per, acc, const, omega2, dt):
    """
    Calculates the acceleration, velocities and displacement time series for the SDOF oscillator

    Args:
        num_steps (int): number of input motion steps
        num_per (int): number of periods where values are calculated
        acc (np.array): acceleration values
        const (dict): constants of the algorithm
        omega2 (np.array): squared circular frequency
        dt (float): time increment

    Returns:
        tuple: a tuple containing x_a, x_v, x_d
            - x_a = Acceleration time series
            - x_v = Velocity time series
            - x_d = Displacement time series
    """

    x_d = np.zeros([num_steps - 1, num_per], dtype=float)
    x_v = np.zeros_like(x_d)
    x_a = np.zeros_like(x_d)

    for k in range(0, num_steps - 1):
        dug = acc[k + 1] - acc[k]
        z_1 = const['f2'] * dug
        z_2 = const['f2'] * acc[k]
        z_3 = const['f1'] * dug
        z_4 = z_1 / dt
        if k == 0:
            b_val = z_2 - z_3
            a_val = (const['f5'] * b_val) + (const['f4'] * z_4)
        else:
            b_val = x_d[k - 1, :] + z_2 - z_3
            a_val = (const['f4'] * x_v[k - 1, :]) + \
                    (const['f5'] * b_val) + (const['f4'] * z_4)

        x_d[k, :] = (a_val * const['g1']) + (b_val * const['g2']) + \
                    z_3 - z_2 - z_1
        x_v[k, :] = (a_val * const['h1']) - (b_val * const['h2']) - z_4
        x_a[k, :] = (-const['f6'] * x_v[k, :]) - (omega2 * x_d[k, :])

    return x_a, x_v, x_d

@dataclass
class Accelerogram:
    """ Acceleration record.

    Gets acceleration values and dt and calculates several motion parameters.

    Most properties are cached, there was no actual need for that since calculations do not really
    take such a long time. Used the `cached-property package <https://pypi.org/project/cached-property/>`_.

    .. uml::

        class Record {
        .. initialize attributes ..
        + accelerations: np.array
        + dt: float
        .. cached properties ..
        + steps
        + time_total
        + time
        + velocities
        + displacements
        + props
        + husid
        + husid_norm
        .. properties ..
        + output
        + plot_time_series
        .. classmethods - cunstructors ..
        + from_one_column_txt(filename, dt)
        + from_multi_column_txt(filename, dt, skip_header=0, skip_footer=0)
        + from_karakostas_acc(filename, direction)
        .. methods ..
        + significant_duration(start_percentage, end_percentage)

        }

    Attributes:
        accelerations (np.array): an array with the acceleration values. Units should be m/sec2
        dt (float): time increment

    """
    accelerations: np.array
    dt: float

    @classmethod
    def from_one_column_txt(cls, filename, dt, scale=1.0):
        """constructor that loads values from a single column text file"""
        accel = np.loadtxt(filename)
        return cls(dt=dt, accelerations=accel*scale)

    @classmethod
    def from_multi_column_txt(cls, filename, dt, scale=1.0, skip_header=0, skip_footer=0):
        """constructor that loads values from a multicolumn text file

            Args:
                filename: file to load
                dt: time increment
                skip_header (int): number of lines to skip at the beginning of the file
                skip_footer (int): number of lines to skip at the end of the file

        """
        accel = np.genfromtxt(fname=filename, skip_header=skip_header, skip_footer=skip_footer)
        return cls(dt=dt, accelerations=accel.flatten()*scale)

    @classmethod
    def from_athanassiadou_acc(cls, filename, direction, scale=0.01):
        """direction = 'L' or 'T' or 'V'"""
        f_df = pd.read_csv(filename, sep=r"\s+")
        dt = f_df['Time(s)'][1] - f_df['Time(s)'][0]
        col_name = 'Acc' + direction + '(cm/s2)'
        accel = np.array(f_df[col_name].tolist())
        return cls(dt=dt, accelerations=accel*scale)

    def significant_duration(self, start_percentage, end_percentage):
        # self.props['T_Arias5_95'] = self.significant_duration(0.05, 0.95)
        t_start = float(np.interp(start_percentage, self.husid_norm, self.time))
        t_end = float(np.interp(end_percentage, self.husid_norm, self.time))
        return t_end - t_start

    def get_spectra(self, damping, startT=0., endT=4., noTs=401):
        _spec = Spectra(accel=self.accelerations,
                        dt=self.dt)
        return _spec.get_spectra(damping, startT, endT, noTs)

    def show_parameters(self):
        paramlog = []
        paramlog.append('PGA = {:.4f}'.format(self.props['PGA']))
        paramlog.append('time for PGA = {:.4f}'.format(self.props['timePGA']))
        paramlog.append('PGV = {:.4f}'.format(self.props['PGV']))
        paramlog.append('time for PGV = {:.4f}'.format(self.props['timePGV']))
        paramlog.append('PGD = {:.4f}'.format(self.props['PGD']))
        paramlog.append('time for PGD = {:.4f}'.format(self.props['timePGD']))
        paramlog.append('PGV/PGA = {:.4f}'.format(self.props['PGVdivPGA']))
        paramlog.append('aRMS = {:.4f}'.format(self.props['aRMS']))
        paramlog.append('vRMS = {:.4f}'.format(self.props['vRMS']))
        paramlog.append('dRMS = {:.4f}'.format(self.props['dRMS']))
        paramlog.append('Arias Intensity Ia = {:.4f}'.format(self.props['Ia']))
        paramlog.append('Characteristic Intensity Ic = {:.4f}'.format(self.props['Ic']))
        paramlog.append('Specific Energy Density SED = {:.4f}'.format(self.props['SED']))
        paramlog.append('Cumulative Absolute Velocity CAV = {:.4f}'.format(self.props['CAV']))
        # paramlog.append('Significant duration (5-95% in Husid plot) = {:.4f}'.format(self.props['T_Arias5_95']))

        return '\n'.join(paramlog)


    def clear_cached_properties(self):
        del self.__dict__['steps']
        del self.__dict__['time_total']
        del self.__dict__['time']
        del self.__dict__['velocities']
        del self.__dict__['displacements']
        del self.__dict__['props']
        del self.__dict__['husid']
        del self.__dict__['husid_norm']

    @cached_property
    def steps(self):
        """int: number of steps"""
        return len(self.accelerations)

    @cached_property
    def time_total(self):
        """float: total time of the motion"""
        return (self.steps - 1) * self.dt

    @cached_property
    def time(self):
        """np.array: time values"""
        return np.arange(0., self.time_total + 1e-6, self.dt)

    @cached_property
    def velocities(self):
        """np.array: velocity values"""
        return self.dt * integrate.cumtrapz(self.accelerations, initial=0.)

    @cached_property
    def displacements(self):
        """np.array: displacement values"""
        return self.dt * integrate.cumtrapz(self.velocities, initial=0.)

    @cached_property
    def props(self):
        """dict: a dictionary with several input motion parameters"""
        _ppp = dict()

        _ppp['PGA'] = max(np.absolute(self.accelerations))
        _ppp['PGV'] = max(np.absolute(self.velocities))
        _ppp['PGD'] = max(np.absolute(self.displacements))
        _ppp['timePGA'] = float(np.where(np.absolute(self.accelerations) == _ppp['PGA'])[0][0]) * self.dt
        _ppp['timePGV'] = float(np.where(np.absolute(self.velocities) == _ppp['PGV'])[0][0]) * self.dt
        _ppp['timePGD'] = float(np.where(np.absolute(self.displacements) == _ppp['PGD'])[0][0]) * self.dt
        _ppp['PGVdivPGA'] = _ppp['PGV'] / _ppp['PGA']
        _ppp['aRMS'] = math.sqrt((1. / self.time_total) * integrate.trapz(self.accelerations * self.accelerations, self.time))
        _ppp['vRMS'] = math.sqrt(
            (1. / self.time_total) * integrate.trapz(self.velocities * self.velocities, self.time))
        _ppp['dRMS'] = math.sqrt(
            (1. / self.time_total) * integrate.trapz(self.displacements * self.displacements, self.time))
        _ppp['Ia'] = (math.pi / (2 * 9.81)) * integrate.trapz(self.accelerations * self.accelerations, self.time)
        _ppp['Ic'] = _ppp['aRMS'] ** 1.5 * math.sqrt(self.time_total)
        _ppp['SED'] = integrate.trapz(self.velocities * self.velocities, self.time)
        _ppp['CAV'] = integrate.trapz(np.absolute(self.accelerations), self.time)

        return _ppp

    @cached_property
    def husid(self):
        """np.array: values of the Husid plot"""
        return math.pi / (2 * 9.81) * integrate.cumtrapz(self.accelerations * self.accelerations, self.time, initial=0.)

    @cached_property
    def husid_norm(self):
        """np.array: values of the normalized Husid plot"""
        return self.husid / self.props['Ia']

    @property
    def output(self):
        """OutputExtended: A collection of output tables and strings

        Tables:
            - RecordMotionProperties
        """
        _output = OutputExtended()
        _output.outputTables['RecordMotionProperties'] = OutputTable()
        _output.outputTables['RecordMotionProperties'].data = convert_dict_to_quantity_value(self.props)

        return _output

    @property
    def plot_time_series(self):
        """Plots acc, vel, dipl vs time"""
        f, ax = plt.subplots(3, figsize=(14, 12))
        ax[0].plot(self.time, self.accelerations / 9.81, label="accelerations", lw=1, color='blue')
        ax[0].set_ylabel('acceleration (g)', fontsize=12)
        # ax[0].set_xlabel('time (sec)', fontsize=12)
        ax[0].legend(fontsize=12)
        ax[1].plot(self.time, self.velocities * 100, label="velocities", lw=1, color='green')
        ax[1].set_ylabel('velocity (cm/sec)', fontsize=12)
        # ax[1].set_xlabel('time (sec)', fontsize=12)
        ax[1].legend(fontsize=12)
        ax[2].plot(self.time, self.displacements * 100, label="displacements", lw=1, color='red')
        ax[2].set_ylabel('displacement (cm)', fontsize=12)
        ax[2].set_xlabel('time (sec)', fontsize=12)
        ax[2].legend(fontsize=12)
        fig = (f, ax)
        return fig

    def get_spectra(self, damping, startT=0., endT=4., noTs=401):
        T = np.linspace(startT, endT, noTs)
        # self.spectrum = NigamJennings(self.accelerations, T, damping, self.dt)
        return NigamJennings(self.accelerations, T, damping, self.dt)

    @staticmethod
    def show_spectra_parameters(spectrum):
        paramlog = []
        paramlog.append('Acceleration Spectrum Intensity: ASI = {:.4f}'.format(spectrum['ASI']))
        paramlog.append('Velocity Spectrum Intensity: VSI = {:.4f}'.format(spectrum['VSI']))
        paramlog.append('Housner Intensity: HI = {:.4f}'.format(spectrum['HI']))
        paramlog.append('Predominant Period: Tp = {:.4f}'.format(spectrum['Tp']))

        return '\n'.join(paramlog)





@dataclass
class Spectra:
    """ A class for calculating the response spectra using the Nigam Jennings approach.
    Adopted code found at the
    `The GEMScienceTools - Risk Modellers Toolkit (RMTK) <https://github.com/GEMScienceTools/rmtk>`_.

    .. uml::

        class Record {
        .. initialize attributes ..
        + accelerations: np.array
        + dt: float

        .. classmethods ..
        + calculate_time_series()
        + NigamJennings()

        .. staticmethods ..
        + show_spectra_parameters

        .. methods ..
        + get_spectra

        }

    Attributes:
        accelerations (np.array): an array with the acceleration values. Units should be m/sec2
        dt (float): time increment
    """
    accel: np.array  # = np.array([])
    dt: float  # = None

    @classmethod
    def calculate_time_series(cls, num_steps, num_per, acc, const, omega2, dt):
        """
        Calculates the acceleration, velocities and displacement time series for the SDOF oscillator

        Args:
            num_steps (int): number of input motion steps
            num_per (int): number of periods where values are calculated
            acc (np.array): acceleration values
            const (dict): constants of the algorithm
            omega2 (np.array): squared circular frequency
            dt (float): time increment

        Returns:
            tuple: a tuple containing x_a, x_v, x_d
                - x_a = Acceleration time series
                - x_v = Velocity time series
                - x_d = Displacement time series
        """

        x_d = np.zeros([num_steps - 1, num_per], dtype=float)
        x_v = np.zeros_like(x_d)
        x_a = np.zeros_like(x_d)

        for k in range(0, num_steps - 1):
            dug = acc[k + 1] - acc[k]
            z_1 = const['f2'] * dug
            z_2 = const['f2'] * acc[k]
            z_3 = const['f1'] * dug
            z_4 = z_1 / dt
            if k == 0:
                b_val = z_2 - z_3
                a_val = (const['f5'] * b_val) + (const['f4'] * z_4)
            else:
                b_val = x_d[k - 1, :] + z_2 - z_3
                a_val = (const['f4'] * x_v[k - 1, :]) + \
                        (const['f5'] * b_val) + (const['f4'] * z_4)

            x_d[k, :] = (a_val * const['g1']) + (b_val * const['g2']) + \
                        z_3 - z_2 - z_1
            x_v[k, :] = (a_val * const['h1']) - (b_val * const['h2']) - z_4
            x_a[k, :] = (-const['f6'] * x_v[k, :]) - (omega2 * x_d[k, :])

        return x_a, x_v, x_d

    @classmethod
    def NigamJennings(cls, acc, periods, damping, dt):
        add_PGA = False
        if periods[0] == 0:
            periods = np.delete(periods, 0)
            add_PGA = True

        num_steps = len(acc)
        num_per = len(periods)
        omega = (2. * np.pi) / np.array(periods)
        omega2 = omega ** 2.
        omega3 = omega ** 3.
        omega_d = omega * math.sqrt(1.0 - (damping ** 2.))
        const = {'f1': (2.0 * damping) / (omega3 * dt),
                 'f2': 1.0 / omega2,
                 'f3': damping * omega,
                 'f4': 1.0 / omega_d}
        const['f5'] = const['f3'] * const['f4']
        const['f6'] = 2.0 * const['f3']
        const['e'] = np.exp(-const['f3'] * dt)
        const['s'] = np.sin(omega_d * dt)
        const['c'] = np.cos(omega_d * dt)
        const['g1'] = const['e'] * const['s']
        const['g2'] = const['e'] * const['c']
        const['h1'] = (omega_d * const['g2']) - (const['f3'] * const['g1'])
        const['h2'] = (omega_d * const['g1']) + (const['f3'] * const['g2'])
        x_a, x_v, x_d = cls.calculate_time_series(num_steps, num_per, acc, const, omega2, dt)

        spectrum = {'Sa': None,
                    'Sv': None,
                    'Sd': None,
                    'T': None,
                    'PSv': None,
                    'PSa': None,
                    'ASI': None,
                    'VSI': None,
                    'HI': None,
                    'Tp': None}

        spectrum['Sa'] = np.max(np.fabs(x_a), axis=0)
        spectrum['Sv'] = np.max(np.fabs(x_v), axis=0)
        spectrum['Sd'] = np.max(np.fabs(x_d), axis=0)
        spectrum['PSv'] = spectrum['Sd'] * omega
        spectrum['PSa'] = spectrum['Sd'] * omega2
        spectrum['T'] = periods

        if add_PGA:
            pga = np.max(np.fabs(acc))
            periods = np.append(0, periods)
            spectrum['Sa'] = np.append(pga, spectrum['Sa'])
            spectrum['Sv'] = np.append(0, spectrum['Sv'])
            spectrum['Sd'] = np.append(0, spectrum['Sd'])
            spectrum['PSv'] = np.append(0, spectrum['PSv'])
            spectrum['PSa'] = np.append(pga, spectrum['PSa'])
            spectrum['T'] = periods

        xy_ASI = xy_with_endpoints(spectrum['T'], spectrum['Sa'], 0.1, 0.5)
        spectrum['ASI'] = integrate.trapz(xy_ASI[1], xy_ASI[0])

        xy_VSI = xy_with_endpoints(spectrum['T'], spectrum['Sv'], 0.1, 2.5)
        spectrum['VSI'] = integrate.trapz(xy_VSI[1], xy_VSI[0])

        xy_HI = xy_with_endpoints(spectrum['T'], spectrum['PSv'], 0.1, 2.5)
        spectrum['HI'] = integrate.trapz(xy_HI[1], xy_HI[0])

        spectrum['Tp'] = (float(np.where(spectrum['Sa'] == max(spectrum['Sa']))[0]) + 1) * (periods[1] - periods[0])

        return spectrum

    @staticmethod
    def show_spectra_parameters(spectrum):
        paramlog = []
        paramlog.append('Acceleration Spectrum Intensity: ASI = {:.4f}'.format(spectrum['ASI']))
        paramlog.append('Velocity Spectrum Intensity: VSI = {:.4f}'.format(spectrum['VSI']))
        paramlog.append('Housner Intensity: HI = {:.4f}'.format(spectrum['HI']))
        paramlog.append('Predominant Period: Tp = {:.4f}'.format(spectrum['Tp']))

        return '\n'.join(paramlog)

    def get_spectra(self, damping, startT=0., endT=4., noTs=401):
        T = np.linspace(startT, endT, noTs)
        # self.spectrum = NigamJennings(self.accelerations, T, damping, self.dt)
        return self.NigamJennings(self.accel, T, damping, self.dt)






from dataclasses import dataclass
from streng.ppp.sections.geometry.rectangular import RectangularSectionGeometry
from streng.ppp.sections.concrete.reinforcement import rectangular_reinforcement
from streng.ppp.sections.concrete.reinforcement import ratios as reinf_ratios
from .common import SectionMaterials, ReinforcementRatios


@dataclass
class RectangularConcreteSection(RectangularSectionGeometry):
    reinforcement: rectangular_reinforcement.RectangularSectionReinforcement
    materials: SectionMaterials

    def __post_init__(self):
        # super(RectangularConcreteSection, self).__post_init__()
        self.calc()


    def calc(self):
        # super(RectangularConcreteSection, self).calc()
        self.d = self.h - self.reinforcement.d1
        self.ratios = ReinforcementRatios(
              ρ1=reinf_ratios.ρ(self.reinforcement.long1.As, self.b, self.d),
              ρ2=reinf_ratios.ρ(self.reinforcement.long2.As, self.b, self.d),
              ρv=reinf_ratios.ρ(self.reinforcement.longV.As, self.b, self.d),
              ρd=reinf_ratios.ρ(self.reinforcement.disdia.As, self.b, self.d),
              ρtot=reinf_ratios.ρ(self.reinforcement.Astot, self.b, self.d),
              ρw=reinf_ratios.ρw(self.reinforcement.trans.n, self.reinforcement.trans.dia, self.b, self.reinforcement.trans.s),
              ω1=reinf_ratios.ω(self.reinforcement.long1.As, self.b, self.d, self.materials.fy, self.materials.fc),
              ω2=reinf_ratios.ω(self.reinforcement.long2.As, self.b, self.d, self.materials.fy, self.materials.fc),
              ωv=reinf_ratios.ω(self.reinforcement.longV.As, self.b, self.d, self.materials.fy, self.materials.fc),
              ωd=reinf_ratios.ω(self.reinforcement.disdia.As, self.b, self.d, self.materials.fy, self.materials.fc),
              ωtot=reinf_ratios.ω(self.reinforcement.Astot, self.b, self.d, self.materials.fy, self.materials.fc))



import xlwings as xw
import math

@xw.func
def test_test(x, y):
    return 5 * (x + y)


@xw.func
def xwf_math_linter(x0, x1, y0, y1, x):
    return y0 + (y1-y0)*(x-x0)/(x1-x0)


@xw.func
@xw.arg('x', doc='This is x.')
@xw.arg('y', doc='This is y.')
def xwf_math_double_sum(x, y):
    """Returns twice the sum of the two arguments"""
    return 2 * (x + y)





from dataclasses import dataclass, field
from streng.common.io.output import OutputTable
from .rectangular import RectangularSectionGeometry as Rect


@dataclass
class TeeSectionGeometry:
    bw: float
    h: float
    beff: float
    hf: float

    _area: float = field(init=False, repr=False)
    _y_g: float = field(init=False, repr=False)
    _x_g: float = field(init=False, repr=False)


    def __post_init__(self):
        self.calc()

    def calc(self):
        self._area = self.bw * (self.h - self.hf) + self.beff * self.hf
        self._y_g = (1.0 / self._area) * ((self.bw * self.h ** 2 / 2.0) + ((self.beff - self.bw) * self.hf ** 2 / 2.0))
        self._x_g = self.beff / 2.0


    @property
    def area(self) -> float:
        return self._area

    @property
    def moment_of_inertia_yy(self) -> float:
        return (self.h - self.hf) * self.bw ** 3 / 12 + self.hf * self.beff ** 3 / 12

    @property
    def moment_of_inertia_xx(self) -> float:
        Ix0 = self.bw * self.h ** 3 / 3.0 + (self.beff - self.bw) * self.hf ** 3 / 3.0
        return Ix0 - self._area * self._y_g ** 2

    @property
    def x_g(self) -> float:
        return self._x_g

    @property
    def y_g(self) -> float:
        """Υπολογισμένο ξεκινώντας από την πλευρά της πλάκας.
        https://calcresource.com/cross-section-tee.html"""
        return self._y_g

    @property
    def torsional_constant(self) -> float:
        """Δεν το έχω υπολογίσει ακόμα...κρατώ αποτέλεσμα ορθογωνικής δοκού"""
        r = Rect(self.bw, self.h)
        return r.torsional_constant
        # return self.bw ** 3 * self.h * (1 / 3 - 0.21 * self.bw / self.h * (1 - self.bw ** 4 / (12 * self.h ** 4)))

    @property
    def shear_area_2(self) -> float:
        return self.bw * self.h

    @property
    def shear_area_3(self) -> float:
        return 5. / 6. * self.beff * self.hf

    @property
    def section_modulus_2(self) -> float:
        return self.moment_of_inertia_yy / self.x_g

    @property
    def section_modulus_3(self) -> float:
        return self.moment_of_inertia_xx / (self.h- self._y_g)

    @property
    def plastic_modulus_2(self) -> float:
        return (self.beff**2 * self.hf / 4.0) + ((self.h - self.hf)*self.bw**2)/4.0

    @property
    def plastic_modulus_3(self) -> float:
        """Δεν το έχω υπολογίσει ακόμα...κρατώ αποτέλεσμα ορθογωνικής δοκού"""
        r = Rect(self.bw, self.h)
        return r.plastic_modulus_3
        # _pm3 = 0
        # if self.y_g()>self.hf:
        #     _pm3 = 0
        # else:
        #     _pm3 = 0
        #
        # return _pm3

    @property
    def radius_of_gyration_2(self) -> float:
        return (self.moment_of_inertia_yy/self._area) ** 0.5

    @property
    def radius_of_gyration_3(self) -> float:
        return (self.moment_of_inertia_xx/self._area) ** 0.5


    @property
    def all_quantities(self):
        out = OutputTable()
        out.data.append({'quantity': 'bw', 'value': self.bw})
        out.data.append({'quantity': 'h', 'value': self.h})
        out.data.append({'quantity': 'beff', 'value': self.beff})
        out.data.append({'quantity': 'hf', 'value': self.hf})
        out.data.append({'quantity': 'area', 'value': self.area})
        out.data.append({'quantity': 'Iyy', 'value': self.moment_of_inertia_yy})
        out.data.append({'quantity': 'Ixx', 'value': self.moment_of_inertia_xx})
        out.data.append({'quantity': 'J', 'value': self.torsional_constant})
        out.data.append({'quantity': 'ShearArea2', 'value': self.shear_area_2})
        out.data.append({'quantity': 'ShearArea3', 'value': self.shear_area_3})
        out.data.append({'quantity': 'SectionModulus2', 'value': self.section_modulus_2})
        out.data.append({'quantity': 'SectionModulus3', 'value': self.section_modulus_3})
        out.data.append({'quantity': 'PlasticModulus2', 'value': self.plastic_modulus_2})
        out.data.append({'quantity': 'PlasticModulus3', 'value': self.plastic_modulus_3})
        out.data.append({'quantity': 'RadiusOfGyration2', 'value': self.radius_of_gyration_2})
        out.data.append({'quantity': 'RadiusOfGyration3', 'value': self.radius_of_gyration_3})
        out.data.append({'quantity': 'xG', 'value': self.x_g})
        out.data.append({'quantity': 'yG', 'value': self.y_g})
        return out


def μ(Μ, b, h, fc):
    """

    Args:
        Μ (float): Moment
        b (float): Section width
        h (float): Section depth (or effective depth)
        fc (float): Concrete compressive strength

    Returns:
        float: Normalised moment

    """
    return Μ / (b * h**2 * fc)



def ν(N, b, h, fc):
    """

    Args:
        N (float): Axial force
        b (float): Section width
        h (float): Section depth (or effective depth)
        fc (float): Concrete compressive strength

    Returns:
        float: Normalised axial force

    """
    return N / (b * h * fc)



import pandas as pd
import string
import numpy as np
from dataclasses import dataclass, field
from typing import List, Dict
from streng.codes.greek.eak2000.raw.ch2.seismic_action import spectra
import openpyxl # import load_workbook
from tabulate import tabulate
from ....phd.building_models.model2d_older.slab_loads import *


# D:\MyBooks\Keimeno\Κεφάλαιο3\Υλικό\FrameAnalyzer\ExcelFiles001\Frames_initial

@dataclass
class Model2d:
    """
    Reads building data from an excel workbook

    ..
        .. http://mdp.tylingsoft.com/#class-diagram
        .. https://media.readthedocs.org/pdf/brandons-sphinx-tutorial/latest/brandons-sphinx-tutorial.pdf
        .. mermaid::

            classDiagram
            Class01 <|-- AveryLongClass : Cool
            Class03 *-- Class04
            Class05 o-- Class06
            Class01 : input_excel_filename (str)
            Class01 : input_data (dict)
            Class01 : input_info (dict)


    Args:
        input_excel_filename (str): Path and filename of the excel input file
        input_data (dict): A dictionary of pandas dataframes with data read from excel worksheets
        input_info (dict): A dictionary with additional input data given in a worksheet named `info`

        stories_total (int): Number of stories (read from joint_coordinates)
        stories_levels (list): A list with storey levels e.g. [4.5, 7.5, 10.5, ...] (read from joint_coordinates)
        stories_heights (list): A list with storey heights e.g. [4.5, 3.0, 3.0, ...] (read from stories_levels)
        x_levels (list): A list with x coordinates of the building (read from joint_coordinates)


    """
    input_excel_filename: str
    input_data: Dict[str, pd.DataFrame] = field(default_factory=dict)
    input_info: Dict = field(default_factory=dict)

    stories_total: int = None
    stories_levels: List[float] = field(default_factory=list)
    stories_heights: List[float] = field(default_factory=list)
    x_levels: List[float] = field(default_factory=list)


    def __post_init__(self):
        """
        Calls the functions that reads the excel file and process model properties
        """
        self.read_excel()
        self.process_joint_coordinates()
        self.process_node_slab_connectivity()
        self.process_beam_slab_connectivity()
        self.process_storey_masses()


    def read_excel(self):
        # info worksheet
        wb = openpyxl.load_workbook(filename=self.input_excel_filename, read_only=True)
        i = 1
        while wb['info'][f'B{i}'].value != None:
            self.input_info[ wb['info'][f'A{i}'].value] = wb['info'][f'B{i}'].value
            i+=1

        self.input_data['joint_coordinates'] = pd.read_excel(io=self.input_excel_filename,
                                                             sheet_name='dtNodes',
                                                             index_col='NodeID')

        self.input_data['materials_concrete'] = pd.read_excel(io=self.input_excel_filename,
                                                              sheet_name='dtConcrete')
        self.input_data['materials_steel'] = pd.read_excel(io=self.input_excel_filename,
                                                           sheet_name='dtSteel')

        self.input_data['sections'] = pd.read_excel(io=self.input_excel_filename,
                                                  sheet_name='dtSections',
                                                  index_col = 'SectionName')

        self.input_data['elements'] = pd.read_excel(io=self.input_excel_filename,
                                                  sheet_name='dtElements',
                                                  index_col = 'ElementID')

        self.input_data['node_slab_connectivity'] = pd.read_excel(io=self.input_excel_filename,sheet_name='dtNodeSlabConn',
                                                                  index_col = 'NodeID').query('Storey>=1').fillna('')

        self.input_data['beam_slab_connectivity'] = pd.read_excel(io=self.input_excel_filename,sheet_name='dtBeamSlabConn',
                                                               index_col = 'ElementID').fillna('')



    def process_joint_coordinates(self):
        self.stories_total = max(self.input_data['joint_coordinates']['Storey'].unique())
        self.stories_levels = self.input_data['joint_coordinates']['Z'].unique().tolist()
        self.x_levels = self.input_data['joint_coordinates']['X'].unique().tolist()

        st_h = []
        for i in range(0,self.stories_total):
            st_h.append(self.stories_levels[i+1] - self.stories_levels[i])
        self.stories_heights = st_h


    def process_node_slab_connectivity(self):

        _df_input = self.input_data['node_slab_connectivity']

        _df = pd.DataFrame(columns=['NodeID'])

        # import general info
        _building_type = self.input_info['building_type']
        _γσκυρ = self.input_info['γσκυρ']
        _gπλ = self.input_info['gπλ']
        _qπλ = self.input_info['qπλ']
        _qπλ = self.input_info['qπλ']
        _hf = self.input_info['hf']
        _gτοιχ_in = self.input_info['gτοιχ_in']
        _gτοιχ_out = self.input_info['gτοιχ_out']
        _hτοιχ = self.input_info['hτοιχ']
        _b_beam_y = self.input_info['b_beam_y']
        _h_beam_y = self.input_info['h_beam_y']
        _L_beam_y = self.input_info['L_beam_y']
        _h_storey = self.input_info['h_storey']


        for index, row in _df_input.iterrows():
            # import from excel table
            _NodeID = index
            _storey = row['Storey']
            _pointEtabs = row['Point_Etabs']
            _frame_trans_loc = row['Frame_Trans_Loc']
            _slabConnΔ1 = row['SlabConΔ1']
            _slabConnΔ2 = row['SlabConΔ2']

            # Ίδιο βάρος στύλου
            if _storey != self.stories_total:
                i = self.input_data['elements'].query(
                    '(NodeI==@_NodeID) & (ElementType!="BEAM")').first_valid_index()
                _col_geom_name = self.input_data['elements']['GeometryName'][i]
                _b_col, _h_col = self.get_element_b_h(_col_geom_name)
            else:
                _col_geom_name = ''
                _b_col, _h_col = 0., 0.

            # Ίδιο βάρος τοιχοπληρώσεων
            if _frame_trans_loc == 'IN':
                _gτοιχ = _gτοιχ_in
            else:
                _gτοιχ = _gτοιχ_out

            # Φορτία δοκών
            if _slabConnΔ1 != '':
                _gΔ1πλ = phd_beam_loads_from_slabs(_building_type, _slabConnΔ1, _gπλ)
                _qΔ1 = phd_beam_loads_from_slabs(_building_type, _slabConnΔ1, _qπλ)
                _gΔ1ΙΒ = _γσκυρ * _b_beam_y * (_h_beam_y - _hf)
                if _storey != self.stories_total:
                    _gΔ1τοιχ = _gτοιχ * _hτοιχ
                else:
                    _gΔ1τοιχ = 0.0
                _gΔ1ολ = _gΔ1πλ + _gΔ1ΙΒ + _gΔ1τοιχ
            else:
                _gΔ1πλ = 0.
                _qΔ1 = 0.
                _gΔ1ΙΒ = 0.
                _gΔ1τοιχ = 0.
                _gΔ1ολ = 0.

            if _slabConnΔ2 != '':
                _gΔ2πλ = phd_beam_loads_from_slabs(_building_type, _slabConnΔ2, _gπλ)
                _qΔ2 = phd_beam_loads_from_slabs(_building_type, _slabConnΔ2, _qπλ)
                _gΔ2ΙΒ = _γσκυρ * _b_beam_y * (_h_beam_y - _hf)
                if _storey != self.stories_total:
                    _gΔ2τοιχ = _gτοιχ * _hτοιχ
                else:
                    _gΔ2τοιχ = 0.0
                _gΔ2ολ = _gΔ2πλ + _gΔ2ΙΒ + _gΔ2τοιχ
            else:
                _gΔ2πλ = 0.
                _qΔ2 = 0.
                _gΔ2ΙΒ = 0.
                _gΔ2τοιχ = 0.
                _gΔ2ολ = 0.

            _G = (_gΔ1ολ + _gΔ2ολ) * _L_beam_y / 2.0 + _γσκυρ * _b_col * _h_col * _h_storey
            _Q = (_qΔ1 + _qΔ2) * _L_beam_y / 2.0


            _df = _df.append({'NodeID': _NodeID,
                              'Storey': _storey,
                              'Point_Etabs': _pointEtabs,
                              'ColGeom': _col_geom_name,
                              'gπλ': _gπλ,
                              'qπλ': _qπλ,
                              'hf': _hf,
                              'gτοιχ': _gτοιχ,
                              'hτοιχ': _hτοιχ,
                              'b_col': _b_col,
                              'h_col': _h_col,
                              'b_beam_y': _b_beam_y,
                              'h_beam_y': _h_beam_y,
                              'gΔ1πλ': _gΔ1πλ,
                              'qΔ1': _qΔ1,
                              'gΔ2πλ': _gΔ2πλ,
                              'qΔ2': _qΔ2,
                              'gΔ1ΙΒ': _gΔ1ΙΒ,
                              'gΔ1τοιχ': _gΔ1τοιχ,
                              'gΔ1': _gΔ1ολ,
                              'gΔ2ΙΒ': _gΔ2ΙΒ,
                              'gΔ2τοιχ': _gΔ2τοιχ,
                              'gΔ2': _gΔ2ολ,
                              'G': _G,
                              'Q': _Q,
                              },
                              ignore_index=True)

        self.input_data['node_loads'] = _df


    def process_beam_slab_connectivity(self):
        _df_input = self.input_data['beam_slab_connectivity']

        _df = pd.DataFrame(columns=['ElementID',
                                    'GeometryName'])

        # import general info
        _building_type = self.input_info['building_type']
        _γσκυρ = self.input_info['γσκυρ']
        _gπλ = self.input_info['gπλ']
        _qπλ = self.input_info['qπλ']
        _qπλ = self.input_info['qπλ']
        _hf = self.input_info['hf']
        _gτοιχ_in = self.input_info['gτοιχ_in']
        _gτοιχ_out = self.input_info['gτοιχ_out']
        _hτοιχ = self.input_info['hτοιχ']

        for index, row in _df_input.iterrows():
            # import from excel table
            _elementID = index
            _geom_name = row['GeometryName']
            _storey = row['Storey']
            _lineEtabs = row['Line_Etabs']
            _slabConn = row['SlabConnectivity']
            _frameLoc = row['Frame_Loc']

            if _frameLoc == 'IN':
                _gτοιχ = _gτοιχ_in
            else:
                _gτοιχ = _gτοιχ_out

            _b, _h = self.get_element_b_h(_geom_name)

            _gΔΙΒ = _γσκυρ * _b * (_h - _hf)

            if _storey != self.stories_total:
                _gΔτοιχ = _gτοιχ * _hτοιχ
            else:
                _gΔτοιχ = 0.0

            _gΔπλ = phd_beam_loads_from_slabs(_building_type, _slabConn, _gπλ)
            _qΔ = phd_beam_loads_from_slabs(_building_type, _slabConn, _qπλ)

            _gΔολ = _gΔπλ + _gΔΙΒ + _gΔτοιχ


            _df = _df.append({'ElementID': _elementID,
                              'GeometryName': _geom_name,
                              'Storey': _storey,
                              'Line_Etabs': _lineEtabs,
                              'SlabConnectivity': _slabConn,
                              'gπλ': _gπλ,
                              'qπλ': _qπλ,
                              'hf': _hf,
                              'gτοιχ': _gτοιχ,
                              'hτοιχ': _hτοιχ,
                              'b': _b,
                              'h': _h,
                              'gΔΙΒ': _gΔΙΒ,
                              'gΔτοιχ': _gΔτοιχ,
                              'gΔπλ': _gΔπλ,
                              'gΔ': _gΔολ,
                              'qΔ': _qΔ},
                              ignore_index=True)

        self.input_data['beam_loads'] = _df


    def process_storey_masses(self):

        m_nodes = []
        _df_input_nodal = self.input_data['node_loads']

        for i in range(0, self.stories_total):
          _g =  _df_input_nodal.query(f'Storey=={i+1}')['G'].sum()
          _q =  _df_input_nodal.query(f'Storey=={i+1}')['Q'].sum()
          m_nodes.append((_g + 0.3 * _q)/9.81)

        m_beams = []
        _df_input_nodal = self.input_data['beam_loads']
        for i in range(0, self.stories_total):
            _load = 0.
            beams_in_storey = _df_input_nodal.query(f'Storey=={i+1}')
            for index, row in beams_in_storey.iterrows():
                _element_id = row['ElementID']
                _gbeam = row['gΔ']
                _qbeam = row['qΔ']
                _element_length = self.get_element_length_by_ID(_element_id)
                _load += (_gbeam + 0.3 * _qbeam) * _element_length

            m_beams.append(_load / 9.81)

        m_tot = [sum(x) for x in zip(m_nodes, m_beams)]

        _data = {'Storey':[i for i in range(1, self.stories_total+1)], 'mass_nodes': m_nodes, 'mass_beams': m_beams, 'mass': m_tot}
        self.input_data['masses'] = pd.DataFrame(data=_data)

    def get_element_length_by_ID(self, elementID):
        _nodeI = self.input_data['elements']['NodeI'][elementID]
        _nodeJ = self.input_data['elements']['NodeJ'][elementID]

        _nodeI_X = self.input_data['joint_coordinates']['X'][_nodeI]
        _nodeI_Y = self.input_data['joint_coordinates']['Y'][_nodeI]
        _nodeI_Z = self.input_data['joint_coordinates']['Z'][_nodeI]

        _nodeJ_X = self.input_data['joint_coordinates']['X'][_nodeJ]
        _nodeJ_Y = self.input_data['joint_coordinates']['Y'][_nodeJ]
        _nodeJ_Z = self.input_data['joint_coordinates']['Z'][_nodeJ]

        return self.get_element_length(_nodeI_X, _nodeI_Y, _nodeI_Z, _nodeJ_X, _nodeJ_Y, _nodeJ_Z)


    @staticmethod
    def get_element_length(x_start, y_start, z_start, x_end, y_end, z_end):
        return ((x_end - x_start)**2 + (y_end - y_start)**2 + (z_end - z_start)**2)**0.5


    def get_element_b_h(self, section_name):
        _b = self.input_data['sections']['b'][section_name]
        _h = self.input_data['sections']['h'][section_name]
        return _b, _h


    def show_tabulate_input(self, input_table_name):
        return tabulate(self.input_data[input_table_name], headers="keys")


from dataclasses import dataclass, field
from typing import List
import numpy as np
from ...raw.ch8 import csm as csm_atc40
from ......common.math.numerical import intersection
from ......common.io.output import OutputTable, OutputExtended
from ......tools.bilin import Bilin, BilinearCurve
from ......codes.eurocodes.ec8.cls.seismic_action import spectra as spec_ec8



@dataclass
class StructureProperties:
    φ: np.ndarray
    m: np.ndarray
    T0: float
    pushover_curve_F: np.ndarray
    pushover_curve_δ: np.ndarray

    def __post_init__(self):
        self.PF1 = csm_atc40.PF1(self.m, self.φ)
        self.α1 = csm_atc40.α1(self.m, self.φ)
        self.φroof1 = self.φ[-1]
        self.Sa = csm_atc40.Sa(V=self.pushover_curve_F,
                               W=sum(self.m),
                               α1=self.α1)
        self.Sd = csm_atc40.Sd(Δroof=self.pushover_curve_δ,
                               PF1=self.PF1,
                               φroof1=self.φroof1)


@dataclass
class Demand:
    T_range: np.ndarray
    Sa: np.ndarray
    Sd: np.ndarray
    TC: float

    def ec8_elastic(self, αgR: float, γI: float, ground_type: str, spectrum_type: int, η=1.0, q=1.0, β=0.2):
        _spec_ec8 = spec_ec8.SpectraEc8(αgR, γI, ground_type, spectrum_type, η, q, β)
        self.Sa = _spec_ec8.Se(self.T_range)
        self.Sd = _spec_ec8.SDe(self.T_range)
        self.TC = _spec_ec8.TC


@dataclass
class N2Method:
    structure: StructureProperties
    demand: Demand

    def __post_init__(self):
        self.output = OutputTable()

    @staticmethod
    def Rμ(μ, T, TC):
        if T < TC:
            return (μ-1)*(T/TC) + 1.0
        else:
            return μ

    @staticmethod
    def Sa(_Sae, _Rμ):
        return _Sae/_Rμ

    @staticmethod
    def Sd(_Sde, _μ, _Rμ):
        return (_μ/_Rμ) * _Sde

    def Sa_for_μ(self, μ):
        _rm = np.array([self.Rμ(μ, t, self.demand.TC) for t in self.demand.T_range])
        _sa = self.demand.Sa / _rm
        return _sa

    def Sd_for_μ(self, μ):
        _rm = np.array([self.Rμ(μ, t, self.demand.TC) for t in self.demand.T_range])
        _sd = self.Sd(self.demand.Sd, μ, _rm)
        return _sd




from dataclasses import dataclass, field
from typing import List
import numpy as np
from ......codes.usa.atc40.raw.ch8 import csm as csm_atc40
# from ...raw.ch8 import csm as csm_atc40
from ......common.math.numerical import intersection
from ......common.io.output import OutputTable, OutputExtended
from ......tools.bilin import Bilin, BilinearCurve
from ......codes.eurocodes.ec8.cls.seismic_action import spectra as spec_ec8


@dataclass
class StructureProperties:
    φ: np.ndarray
    m: np.ndarray
    T0: float
    pushover_curve_F: np.ndarray
    pushover_curve_δ: np.ndarray
    behavior: str

    def __post_init__(self):
        self.PF1 = csm_atc40.PF1(self.m, self.φ)
        self.α1 = csm_atc40.α1(self.m, self.φ)
        self.φroof1 = self.φ[-1]
        self.Sa = csm_atc40.Sa(V=self.pushover_curve_F,
                               W=sum(self.m),
                               α1=self.α1)
        self.Sd = csm_atc40.Sd(Δroof=self.pushover_curve_δ,
                               PF1=self.PF1,
                               φroof1=self.φroof1)


@dataclass
class Demand:
    T_range: np.ndarray
    Sa: np.ndarray
    Sd: np.ndarray
    TC: float

    def ec8_elastic(self, αgR: float, γI: float, ground_type: str, spectrum_type: int, η=1.0, q=1.0, β=0.2):
        _spec_ec8 = spec_ec8.SpectraEc8(αgR, γI, ground_type, spectrum_type, η, q, β)
        self.Sa = _spec_ec8.Se(self.T_range)
        self.Sd = _spec_ec8.SDe(self.T_range)
        self.TC = _spec_ec8.TC


@dataclass
class CapacitySpectrumMethod:
    structure: StructureProperties
    demand: Demand
    first_try_case: str = 'intersection'

    def __post_init__(self):
        self.output = OutputTable()

    @property
    def Sd_first_try(self):
        if self.first_try_case == 'intersection':
            x_solve, y_solve = intersection(self.demand.Sd, self.demand.Sa, self.structure.Sd, self.structure.Sa)
            return x_solve[-1]
        elif self.first_try_case == 'equal displacements':
            Sd_T0 = 100.
            Sa_T0 = Sd_T0 * (2 * np.pi / self.structure.T0) ** 2
            Sa_T0_array = np.array([0.0, Sa_T0])
            Sd_T0_array = np.array([0.0, Sd_T0])
            xT0_solve, yT0_solve = intersection(self.demand.Sd, self.demand.Sa, Sd_T0_array, Sa_T0_array)
            return xT0_solve[-1]
        else:
            return 0.

    def __iterate_SR(self, x0):
        bl = Bilin(xtarget=x0)
        bl.curve_ini.x, bl.curve_ini.y = self.structure.Sd, self.structure.Sa
        bl.calc()

        β0 = bl.bilinear_curve.β0
        βeff = csm_atc40.βeff(0.05, β0, self.structure.behavior)

        Teff = bl.bilinear_curve.Teq(self.structure.T0)


        SRA = csm_atc40.SRA(βeff, self.structure.behavior)
        SRV = csm_atc40.SRV(βeff, self.structure.behavior)

        if Teff > self.demand.TC:
            SR = SRV
        else:
            SR = SRA

        _Sa = SR * self.demand.Sa
        _Sd = SR * self.demand.Sd

        x_solve, y_solve = intersection(_Sd, _Sa, self.structure.Sd, self.structure.Sa)
        return x_solve[-1]

    def calc_performance_point(self):
        # self.output.log_table.dict_list.clear()
        x_i = self.Sd_first_try
        iter_num = 0
        error = 100.
        self.output.data.clear()
        self.output.data.append({'__iteration': iter_num, 'Sd': x_i, 'error': None})
        while error > 0.0001:
            x_new = self.__iterate_SR(x_i)
            error = abs((x_new - x_i) / x_i)
            x_i = x_new
            iter_num = iter_num + 1

            self.output.data.append({'__iteration': iter_num, 'Sd': x_i, 'error': error})

            # print(f'__iteration: {iter_num} x={x_i:.4f} error={error:.2%}')


        print(f'solution: Sd = {x_i:.4f}m')


@dataclass
class CapacitySpectrumMethodProcedureB:
    """
    incomplete...check jupyter notebook
    """
    structure: StructureProperties
    demands: List[Demand] = field(default_factory=list)
    dstar: float = None
    astar: float = None
    bilinear_curve: BilinearCurve = None


    def __post_init__(self):
        self.output = OutputTable()

    def calc_performance_point(self):
        self.dstar_intersection = self._get_dstar()
        self.bilinear_curve = self._get_bilinear_curve()
        self.astar = self.bilinear_curve.au
        self._try_more_points()




    def _get_dstar(self):
        bl = Bilin()
        bl.curve_ini.x = self.structure.Sd
        bl.curve_ini.y = self.structure.Sa
        bl.calc()

        kel = bl.bilinear_curve.kel
        x_kel = np.array([0., 10.])
        y_kel = np.array([0., 10. * kel])

        _dstar, _adstar = intersection(x_kel, y_kel, self.demands[0]['demand'].Sd, self.demands[0]['demand'].Sa)
        self.dstar = _dstar[0]

        return _dstar[0], _adstar[0]


    def _get_bilinear_curve(self):
        bl = Bilin(xtarget=self.dstar)
        bl.curve_ini.x = self.structure.Sd
        bl.curve_ini.y = self.structure.Sa
        bl.calc()
        return bl.bilinear_curve

    def _try_more_points(self):
        self.dpi_rng = np.linspace(0.5 * self.dstar, 1.5 * self.dstar, 11)
        self.api_rng = self._api(self.astar, self.dstar, self.bilinear_curve.ay, self.bilinear_curve.dy, self.dpi_rng)
        self.β0_rng = csm_atc40.β0(self.bilinear_curve.dy, self.bilinear_curve.ay, self.dpi_rng, self.api_rng)
        self.βeff_rng = np.array([csm_atc40.βeff(0.05, x, self.structure.behavior) for x in  self.β0_rng])




    @staticmethod
    def _api(astar, dstar, ay, dy, dpi):
        return ay + (astar - ay) * (dpi - dy) / (dstar - dy)





# -*- coding: utf-8 -*-
import os
import sys
import subprocess
import csv
import numpy

from subprocess import STDOUT, check_output, CalledProcessError

from rmtk import __file__ as rmtk_path
from rmtk.vulnerability.common import utils

# FIXME we should not write data on the python package folder!
# For now it works only when installed with 'pip -e'
bin_directory = os.path.join(os.path.dirname(os.path.abspath(rmtk_path)),
                             '../bin')
package_directory = os.path.dirname(os.path.abspath(__file__))


def calculate_fragility(capacity_curves, hysteresis, msa, gmrs, damage_model,
                        damping, degradation):

    no_damage_states = len(damage_model['damage_states'])
    no_bins = msa['n. bins']
    no_rec_bin = msa['records per bin']
    no_gmrs = no_bins*no_rec_bin
    no_capacity_curves = len(capacity_curves['Sd'])
    PDM = numpy.zeros((no_bins, no_damage_states+1))
    Sds = numpy.zeros((no_gmrs, no_capacity_curves))
    gmr_info = numpy.zeros((no_gmrs, 2))

    for icc in range(no_capacity_curves):
        print str((icc+1)*100/no_capacity_curves) + '%'
        folder_scaled_recs = msa['input folder']
        target_files = os.listdir(folder_scaled_recs)
        for ele in target_files:
            if ele[-3:] != 'csv':
                target_files.remove(ele)
        counter = 0
        for index, ibin in enumerate(target_files):
            if index < no_bins:
                print ibin
            else:
                break

            with open(folder_scaled_recs+'/'+ibin, 'rb') as f:
                reader = csv.reader(f)
                newlist = [row for row in reader]
            record_name = [ele[0] for ele in newlist]
            record_scaling = [float(ele[1]) for ele in newlist]

            for igmr in range(no_rec_bin):
                limit_states = utils.define_limit_states(capacity_curves, icc,
                                                         damage_model)
                target_name = record_name[igmr]
                scaling_factor = record_scaling[igmr]
                time, disps = run_time_history_analysis(capacity_curves,
                                                        hysteresis, icc, gmrs,
                                                        target_name,
                                                        scaling_factor,
                                                        damping,
                                                        degradation)
                Sdi = max(numpy.abs(numpy.array(disps)))
                # with open('dynamicResult.csv', 'rb') as f:
                # reader = csv.reader(f)
                # newlist = [row[0] for row in reader]
                # if newlist == ['KO']:
                #    ds = no_ls
                [PDM, ds] = utils.allocate_damage(index, PDM, Sdi,
                                                  limit_states)
                gmr_info[counter, :] = [gmrs['name'].index(target_name),
                                        scaling_factor]
                Sds[counter][icc] = Sdi
                counter = counter+1
                print "gmr n.", counter, "max disp. =", Sdi, "DS =", ds

    return PDM, Sds, gmr_info


def run_time_history_analysis(capacity_curves, hysteresis, icc, gmrs,
                              target_name, scaling_factor, damping,
                              degradation):

    igmr = gmrs['name'].index(target_name)
    write_gmr(gmrs, igmr, scaling_factor)
    create_SDOF_paramfile(capacity_curves, hysteresis, icc, damping,
                          degradation)

    tcl = os.path.join(package_directory, "simplestSDOFgen.tcl")

    try:
        if 'darwin' in sys.platform:
            check_output([os.path.join(bin_directory,
                         "OpenSees.mac"), tcl],
                         cwd=package_directory, stderr=STDOUT)
        elif 'linux' in sys.platform:
            check_output([os.path.join(bin_directory,
                         "OpenSees.u1604"), tcl],
                         cwd=package_directory, stderr=STDOUT)
        else:
            sys.exit('Your OS is not currently support')
    except CalledProcessError as e:
        print(e.output)
        raise

    disps = []
    time = []
    disp_file = open(os.path.join(package_directory, 'NodeD.out'))
    lines = disp_file.readlines()
    for line in lines:
        values = line.strip().split(' ')
        time.append(float(values[0]))
        disps.append(float(values[1]))

    return time, disps


def write_gmr(gmrs, igmr, scaling_factor):
    out_file = open(os.path.join(package_directory, 'gmr.txt'), 'w')
    out_file.write(str(gmrs['no_points'][igmr])+'\n')
    out_file.write(str(gmrs['dt'][igmr])+'\n')
    for iacc in range(gmrs['no_points'][igmr]):
        out_file.write(str(gmrs['acc'][igmr][iacc]*scaling_factor)+'\n')
    out_file.close()


def create_SDOF_paramfile(capacity_curves, hysteresis, icc, damping,
                          degradation):

    d_flag = 0
    if degradation:
        d_flag = 1

    out_file = open(os.path.join(package_directory, 'SDOF_paramfile.tcl'), 'w')
    out_file.write('set material "pinchgeneric" \n')
    f1 = capacity_curves['Sa'][icc][1]*9.81
    f2 = capacity_curves['Sa'][icc][2]*9.81
    f3 = capacity_curves['Sa'][icc][3]*9.81
    f4 = capacity_curves['Sa'][icc][4]*9.81
    d1 = capacity_curves['Sd'][icc][1]
    d2 = capacity_curves['Sd'][icc][2]
    d3 = capacity_curves['Sd'][icc][3]
    d4 = capacity_curves['Sd'][icc][4]
    d5 = capacity_curves['Sd'][icc][4]
    # -capacity_curves['Sd'][icc][1]
    out_file.write('set fi {%f %f %f %f 0.0} \n' % (f1, f2, f3, f4))
    out_file.write('set di {%f %f %f %f %f} \n' % (d1, d2, d3, d4, d5))
    out_file.write('set ksi %f \n' % damping)
    out_file.write('set Tperiod "0.0" \n')
    out_file.write('set mx "1.0" \n')
    out_file.write('set degflag "%d" \n' % d_flag)
    out_file.write('set analysistype "dynamic" \n')
    out_file.write('set pinchingtype "%s" \n' % hysteresis['type'])

    if hysteresis['type'] == 'user-defined':
        out_file.write('set rDisp %f \n' % hysteresis['rDisp'][0])
        out_file.write('set fForce %f \n' % hysteresis['fForce'][0])
        out_file.write('set uForce %f \n' % hysteresis['uForce'][0])
        gK1, gK2, gK3, gK4, gKLim = hysteresis['gK'][0]
        gD1, gD2, gD3, gD4, gDLim = hysteresis['gD'][0]
        gF1, gF2, gF3, gF4, gFLim = hysteresis['gF'][0]
        out_file.write('set gammaK [list %f %f %f %f %f]\n' % (gK1, gK2, gK3,
                                                               gK4, gKLim))
        out_file.write('set gammaD [list %f %f %f %f %f]\n' % (gD1, gD2, gD3,
                                                               gD4, gDLim))
        out_file.write('set gammaF [list %f %f %f %f %f]\n' % (gF1, gF2, gF3,
                                                               gF4, gFLim))
        out_file.write('set gammaE %f \n' % hysteresis['gE'][0])
        out_file.write('set damage "%s"\n' % hysteresis['dmgType'][0])

    out_file.close()



from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Table, Column, Integer, String, Date, MetaData, Float, ForeignKey
from sqlalchemy.orm import relationship

Base = declarative_base()


class BuildingIdentity(Base):
    __tablename__ = 'Building_Identities'
    id = Column(String, primary_key=True)
    NodesBuilding = Column(String)
    SectionsBuilding = Column(String)

    def __init__(self, id, NodesBuilding, SectionsBuilding):
        self.id = id
        self.NodesBuilding = NodesBuilding
        self.SectionsBuilding = SectionsBuilding


class NodesCase(Base):
    __tablename__ = 'nodes_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class Node(Base):
    __tablename__ = 'nodes'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('nodes_cases.Id'))
    NodeID = Column(Integer)
    X = Column(Float)
    Y = Column(Float)
    Z = Column(Float)
    Storey = Column(Integer)
    Point_Etabs = Column(Integer)

    def __init__(self, Id, CaseId, NodeID, X, Y, Z, Storey, Point_Etabs):
        self.Id = Id
        self.CaseId = CaseId
        self.NodeID = NodeID
        self.X = X
        self.Y = Y
        self.Z = Z
        self.Storey = Storey
        self.Point_Etabs = Point_Etabs

    def __repr__(self):
        return f'{self.CaseId}: Node {self.NodeID} coordinates {self.X}, {self.Y} , {self.Z}'


class SectionsCase(Base):
    __tablename__ = 'sections_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class Section(Base):
    __tablename__ = 'sections'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('sections_cases.Id'))
    ElementType = Column(String)
    Frame = Column(Integer)
    Storey = Column(Integer)
    Frame_Loc = Column(String)
    SectionName = Column(String)
    b = Column(Float)
    h = Column(Float)

    def __init__(self, Id, CaseId, ElementType, Frame, Storey, Frame_Loc, SectionName, b, h):
        self.Id = Id
        self.CaseId = CaseId
        self.ElementType = ElementType
        self.Frame = Frame
        self.Storey = Storey
        self.Frame_Loc = Frame_Loc
        self.SectionName = SectionName
        self.b = b
        self.h = h


class ElementsCase(Base):
    __tablename__ = 'elements_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class Element(Base):
    __tablename__ = 'elements'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('elements_cases.Id'))
    ElementID = Column(Integer)
    NodeI = Column(Integer)
    NodeJ = Column(Integer)
    ElementType = Column(String)
    SectionName = Column(String)
    Storey = Column(Integer)
    Line_Etabs = Column(String)
    ReinforcementID = Column(String)

    def __init__(self, Id, CaseId, ElementID, NodeI, NodeJ, ElementType, SectionName, Storey, Line_Etabs, ReinforcementID):
        self.Id = Id
        self.CaseId = CaseId
        self.ElementID = ElementID
        self.NodeI = NodeI
        self.NodeJ = NodeJ
        self.ElementType = ElementType
        self.SectionName = SectionName
        self.Storey = Storey
        self.Line_Etabs = Line_Etabs
        self.ReinforcementID = ReinforcementID


class SlabsCase(Base):
    __tablename__ = 'slabs_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class Slab(Base):
    __tablename__ = 'slabs'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('slabs_cases.Id'))
    SlabID = Column(Integer)
    SlabType = Column(String)
    Lmax = Column(Float)
    Lmin = Column(Float)

    def __init__(self, Id, CaseId, SlabID, SlabType, Lmax, Lmin):
        self.Id = Id
        self.CaseId = CaseId
        self.SlabID = SlabID
        self.SlabType = SlabType
        self.Lmax = Lmax
        self.Lmin = Lmin


class NodeSlabConnCase(Base):
    __tablename__ = 'node_slab_conns_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class NodeSlabConn(Base):
    __tablename__ = 'node_slab_conns'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('node_slab_conns_cases.Id'))
    NodeID = Column(Integer)
    Frame_Trans_Loc = Column(String)
    SlabConΔ1 = Column(String)
    SlabConΔ2 = Column(String)

    def __init__(self, Id, CaseId, NodeID, Frame_Trans_Loc, SlabConΔ1, SlabConΔ2):
        self.Id = Id
        self.CaseId = CaseId
        self.NodeID = NodeID
        self.Frame_Trans_Loc = Frame_Trans_Loc
        self.SlabConΔ1 = SlabConΔ1
        self.SlabConΔ2 = SlabConΔ2


class BeamSlabConnCase(Base):
    __tablename__ = 'beam_slab_conns_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class BeamSlabConn(Base):
    __tablename__ = 'beam_slab_conns'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('beam_slab_conns_cases.Id'))
    ElementID = Column(Integer)
    SectionName = Column(String)
    Storey = Column(Integer)
    Line_Etabs = Column(String)
    SlabConnectivity = Column(String)
    Frame_Loc = Column(String)

    def __init__(self, Id, CaseId, ElementID, SectionName, Storey, Line_Etabs, SlabConnectivity, Frame_Loc):
        self.Id = Id
        self.CaseId = CaseId
        self.ElementID = ElementID
        self.SectionName = SectionName
        self.Storey = Storey
        self.Line_Etabs = Line_Etabs
        self.SlabConnectivity = SlabConnectivity
        self.Frame_Loc = Frame_Loc


class MaterialsCase(Base):
    __tablename__ = 'materials_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class Material(Base):
    __tablename__ = 'materials'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('materials_cases.Id'))
    materialID = Column(String)
    fc = Column(Float)
    E = Column(Float)
    U = Column(Float)
    A = Column(Float)
    γc = Column(Float)
    fy = Column(Float)
    fyw = Column(Float)

    def __init__(self, Id, CaseId, materialID, fc, E, U, A, γc, fy, fyw):
        self.Id = Id
        self.CaseId = CaseId
        self.materialID	= materialID
        self.fc = fc
        self.E = E
        self.U = U
        self.A = A
        self.γc = γc
        self.fy = fy
        self.fyw = fyw


class BeamReinforcementsCase(Base):
    __tablename__ = 'beam_reinforcements_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class BeamReinforcement(Base):
    __tablename__ = 'beam_reinforcements'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('beam_reinforcements.Id'))
    ReinforcementID = Column(String)
    LongTopLeft = Column(String)
    LongBotLeft = Column(String)
    TransLeft = Column(String)
    LongTopRight = Column(String)
    LongBotRight = Column(String)
    TransRight = Column(String)

    def __init__(self, Id, CaseId, ReinforcementID, LongTopLeft, LongBotLeft, TransLeft, LongTopRight, LongBotRight, TransRight):
        self.Id = Id
        self.CaseId = CaseId
        self.ReinforcementID = ReinforcementID
        self.LongTopLeft = LongTopLeft
        self.LongBotLeft = LongBotLeft
        self.TransLeft = TransLeft
        self.LongTopRight = LongTopRight
        self.LongBotRight = LongBotRight
        self.TransRight = TransRight


class ColumnReinforcementsCase(Base):
    __tablename__ = 'column_reinforcements_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class ColumnReinforcement(Base):
    __tablename__ = 'column_reinforcements'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('column_reinforcements_cases.Id'))
    ReinforcementID = Column(String)
    LongMain = Column(String)
    LongSide = Column(String)
    Trans = Column(String)
    SideHoopLegs = Column(Integer)

    def __init__(self, Id, CaseId, ReinforcementID, LongMain, LongSide, Trans, SideHoopLegs):
        self.Id = Id
        self.CaseId = CaseId
        self.ReinforcementID = ReinforcementID
        self.LongMain = LongMain
        self.LongSide = LongSide
        self.Trans = Trans
        self.SideHoopLegs = SideHoopLegs


class ModFactorsCase(Base):
    __tablename__ = 'mod_factors_cases'
    Id = Column(String, primary_key=True)

    def __init__(self, Id):
        self.Id = Id


class ModFactor(Base):
    __tablename__ = 'mod_factors'
    Id = Column(Integer, primary_key=True)
    CaseId = Column(String, ForeignKey('mod_factors_cases.Id'))
    ElementType = Column(String)
    Area = Column(Float)
    Moment = Column(Float)
    Shear = Column(Float)
    Torsional = Column(Float)

    def __init__(self, Id, CaseId, ElementType, Area, Moment, Shear, Torsional):
        self.Id = Id
        self.CaseId = CaseId
        self.ElementType = ElementType
        self.Area = Area
        self.Moment = Moment
        self.Shear = Shear
        self.Torsional = Torsional



import time
from functools import wraps


def execution_timer(func):
    # from STONE.RIVER.ELEARNING.PYTHON.TOOLS.MODULES.AND.JSON-iLLiTERATE
    # video 00033 Decorators_and_Monkey_Patching_-_Part_2B.mp4
    @wraps(func)
    def time_wrapper(*args, **kwargs):
        begin_time = time.time()
        result = func(*args, **kwargs)
        complete_time = time.time()
        total_time = complete_time - begin_time
        print("The", func.__name__, "method completed in:", total_time)
        return result
    return time_wrapper




from dataclasses import dataclass


@dataclass
class Kappos91:
    fc: float
    fy: float
    bc: float
    s: float
    ρw: float
    α: float = 1.0
    β: float = 1.0
    εc0: float = 0.002
    ρsx: float = 0.003334

    def __post_init__(self):
        self.ωw = self.ρw * self.fy / self.fc
        self.k = 1 + self.α * self.ωw ** self.β
        self.fcc = self.k * self.fc
        self.εcc0 = self.εc0 * self.k * self.k
        self.ε50c = 0.75 * self.ρw * (self.bc / self.s) ** 0.5 + (3. + 0.29 * self.fc / self.k) / (
                    145 * self.fc / self.k - 1000.)
        self.u = 0.5 / (self.ε50c - self.εc0)
        self.εcult = self.εcc0 + (self.k - 0.85) / (self.k * self.u)

        self.ε25c = (0.75 + self.u * self.εcc0) / self.u

        self.fcc_ec8 = self.fc * (1 + 3.7 * (self.α * self.ρsx * self.fy / self.fc)**0.86)

    def σc(self, εc):
        if εc <= self.εcc0:
            _σc = self.fcc * (2 * (εc / self.εcc0) - (εc / self.εcc0) ** 2)
        else:
            _σc = max(self.fcc * (1 - self.u * (εc - self.εcc0)), 0.25 * self.fcc)
        return _σc


@dataclass
class KentPark71:
    fc: float
    bc: float
    s: float
    ρw: float
    nsp: int = 0
    εc0: float = 0.002

    def __post_init__(self):
        self.ε50u = (3. + 0.29 * self.fc) / (145. * self.fc - 1000.)
        self.ε50h = 0.75 * self.ρw * (self.bc / self.s)**0.5
        self.ε50c = self.ε50u + self.ε50h
        self.z = 0.5 / (self.ε50c - self.εc0)
        self.ε20c = self.εc0 + 1.6 * (self.ε50c - self.εc0)

    def σc(self, εc):
        if εc <= self.εc0:
            _σc = self.fc * (2 * (εc / self.εc0) - (εc / self.εc0) ** 2)
        else:
            _σc = max(self.fc * (1 - self.z * (εc - self.εc0)), 0.2 * self.fc)
            if self.nsp > 0 and εc > self.ε50u:
                _σc = 0.
        return _σc



"""
.. uml::

    @startmindmap
    * db_classes
    ** Accellerogram
    ** RecordDescription
    @endmindmap
"""

import io
import zlib
import numpy as np

# from sqlalchemy import create_engine
# from sqlalchemy import orm
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Table, Column, Integer, String, Date, MetaData, Float, ForeignKey, BLOB

Base = declarative_base()


class Accellerogram(Base):
    """
    .. uml::

        class Accellerogram {
        .. tablename ..
        accellerograms
        .. attributes - columns ..
        + accellerogram_id (String):
        + record_id (String):
        + dt (Float):
        + accellerations_compressed (BLOB):
        .. properties ..
        + accelerations()
        .. staticmethods ..
        + compress_accellerations(accelerations)
        }

    """
    __tablename__ = 'accellerogram'

    accellerogram_id = Column(String, primary_key=True)
    record_id = Column(String)
    dt = Column(Float)
    accellerations_compressed = Column(BLOB)

    def __init__(self, accellerogram_id, record_id, dt, accellerations_compressed):
        self.accellerogram_id = accellerogram_id
        self.record_id = record_id
        self.dt = dt
        self.accellerations_compressed = accellerations_compressed

    @property
    def accellerations(self):
        """

        Returns:
            numpy array: decompresses accelleration values
        """
        return np.load(io.BytesIO(zlib.decompress(self.accellerations_compressed)))

    @staticmethod
    def compress_accellerations(accellerations):
        """
            gets a numpy array with all accelleration values and returns it compressed to store in the sqlite database

        Args:
            accellerations (numpy array): accellerogram values

        Returns:
            a zlib compressed object

        """
        f = io.BytesIO()
        np.save(f, accellerations)
        comp = zlib.compress(f.getvalue())
        return comp

    def __repr__(self):
        return f'{self.symbol}: hey'


class RecordDescription(Base):
    """
    .. uml::

        class RecordDescription {
        .. attributes - columns ..
        + record_id: String
        + date: Date
        + latitude: Float
        + longitude: Float
        + magnitude: Float
        + epicentral_distance: Float
        + ground_ec8: String
        }

    """
    __tablename__ = 'records_descriptions'

    record_id = Column(String, primary_key=True)
    date = Column(Date)
    latitude = Column(Float)
    longitude = Column(Float)
    magnitude = Column(Float)
    epicentral_distance = Column(Float)
    ground_ec8 = Column(String)





import math

def PF1(m, φ):
    """ Modal participation factor for the first natural mode

    .. highlight:: python
    .. code-block:: python
       :linenos:


        PF1 = csm_atc40.PF1(m=np.array([39.08, 39.08, 39.08]),
                            φ=np.array([0.0483, 0.0920, 0.1217]))


    Args:
        m (np.array): mass assigned to level i
        φ (np.array): amplitude of mode 1 al level i

    Returns:
        float: Given using the expression:

        .. math::  PF_1 = \dfrac{\sum(m_i \cdot φ_i)}{\sum(m_i \cdot φ_i^2)}
    """
    return sum(m*φ)/sum(m*φ*φ)


def α1(m, φ):
    """ Modal mass coefficient for the first natural mode

    .. highlight:: python
    .. code-block:: python
       :linenos:


        α1 = csm_atc40.α1(m=np.array([39.08, 39.08, 39.08]),
                          φ=np.array([0.0483, 0.0920, 0.1217]))


    Args:
        m (np.array): mass assigned to level i
        φ (np.array): amplitude of mode 1 al level i

    Returns:
        float: Given using the expression:

        .. math::  α_1 = \dfrac{\sum(m_i \cdot φ_i)^2}{\sum(m_i) \cdot \sum(m_i \cdot φ_i^2)}
    """
    return sum(m * φ)**2 / (sum(m)*sum(m*φ*φ))


def Sa(V, W, α1):
    """ The spectral acceleration

    .. highlight:: python
    .. code-block:: python
       :linenos:

        Sa = csm_atc40.Sa(V=bl.y_ini,
                          W=sum(m),
                          α1=α1) # Βάζω όπου W τη μάζα ώστε να βγει το Sa σε m/sec2 και όχι σε g


    Args:
        V (np.array): base shear
        W (np.array): building weight (dead plus likely live loads...eg g+0.3q)
        α1 (float): modal mass coefficient for the first natural mode

    Returns:
        float: Given using the expression:

        .. math::  S_a = \dfrac{V/W}{α_1}
    """
    return (V/W)/α1


def Sd(Δroof, PF1, φroof1):
    return Δroof / (PF1*φroof1)


def T(Sa, Sd):
    return 2*math.pi*(Sd/Sa)**0.5


def β0(dy, ay, dpi, api):
    return (2/math.pi)*(ay*dpi-dy*api)/(api*dpi)


def βeff(β, β0, behavior):
    # Το όριο του 0.45 φαίνεται στa Figure 8-15, 8-16
    if β0 > 0.45:
        β0 = 0.45

    _κ = κ(β0, behavior)
    return β + _κ * β0


def κ(β0, behavior):
    # Το όριο του 0.45 φαίνεται στa Figure 8-15, 8-16
    if β0 > 0.45:
        β0 = 0.45

    if behavior == 'A':
        if β0 <= 0.1625:
            return 1.0
        else:
            return 1.13 - 0.51 * math.pi / 2 * β0
    elif behavior == 'B':
        if β0 <= 0.25:
            return 0.67
        else:
            return 0.845 - 0.446 * math.pi / 2 * β0
    else:
        return 0.33


def SRA(βeff, behavior):
    if behavior == 'A':
        minSRA = 0.33
    elif behavior == 'B':
        minSRA = 0.44
    else:
        minSRA = 0.56

    return max(minSRA, ((3.21 - 0.68 * math.log(100 * βeff)) / 2.12))


def SRV(βeff, behavior):
    if behavior == 'A':
        minSRV = 0.50
    elif behavior == 'B':
        minSRV = 0.56
    else:
        minSRV = 0.69

    return max(minSRV, ((2.31 - 0.41 * math.log(100 * βeff)) / 1.65))


import os
import numpy
import math
from numpy import numarray

def compute_disp(accs, accstep, period, damping):
	
	fraction = 1/0.02

	pNocount = 0
	lfine = 0
	np = len(accs)
	pNocount=pNocount+1
	
	MaxSteps = (np+1) * (round(int(accstep*fraction/period)) + 1) +1
	ugh=numarray.zeros(MaxSteps)

	
	fine = round(int(accstep * fraction / period)) + 1
	if fine != lfine:
		L = 1
		i = 1
		while (i <=1 + (np - 1) * fine ):
			i=i+fine
			L=L+1
	
	
	dt = accstep / fine
	lfine = fine
	M = 1 + (np - 1) * fine

	pNocount = 0
	xie = damping
	maxug = max(accs)
	lfine =0

	pNocount = 1
	fine = round(int(accstep * fraction / period)) + 1
	if fine != lfine:
		L = -1
		i = 0
		while (i <= (np - 1) * fine ):
			i=i+fine
			L=L+1	
			ugh[i] = accs[L]
			for M in range(int(fine)):
				ugh[i-M+1] = ugh[i - fine] + (ugh[i] - ugh[i - fine]) * (fine - M +1) / fine

	lfine = fine
	M = 1 + (np - 1) * fine
	dt = accstep / fine
	ncf = 2.0 * 3.14159265 / period
	
	fraction = 1.0/0.02
	Gamma_Parm = 0.5
	Beta_Parm = 0.25
	damp = 0.05
	THdsps = []
	THaccs = []
	U0 = 0.0
	U1 = 0.0
	V0 = 0.0
	V1 = 0.0
	A0 = 0.0
	A1 = 0.0
	xie = damping
	for i in range(int(M)):
		U1 = (-ugh[i+1] + U0 / Beta_Parm / (dt*dt) + V0 / Beta_Parm / dt + (1.0 / 2.0 / Beta_Parm - 1.0) * A0 + (U0 * Gamma_Parm / Beta_Parm / dt + (Gamma_Parm / Beta_Parm - 1.0) * V0 + (Gamma_Parm / 2.0 / Beta_Parm - 1.0) * dt * A0) * 2.0 * xie * ncf) / (1.0 / Beta_Parm / (dt*dt) + 2.0 * xie * ncf * Gamma_Parm / Beta_Parm / dt + (ncf*ncf))
		V1 = (U1 - U0) * Gamma_Parm / Beta_Parm / dt + (1.0 - Gamma_Parm / Beta_Parm) * V0 + (1.0 - Gamma_Parm / 2.0 / Beta_Parm) * dt * A0
		A1 = (U1 - U0) / Beta_Parm / (dt*dt) - V0 / Beta_Parm / dt + (1.0 - 1.0 / 2.0 / Beta_Parm) * A0
		U0 = U1
		V0 = V1
		A0 = A1
		THdsps.append(U1)
		THaccs.append(A1+ugh[i+1])
		
	Sa = max([math.fabs(min(THaccs)),max(THaccs)])
	Sd = max([math.fabs(min(THdsps)),max(THdsps)])*9.81*100	
	
	return Sd,Sa


from streng.codes.eurocodes.ec8.raw.ch5.detailing import beams as detbeams
from streng.common.io import results_display_older
from .....sections.common import norm_forces
from .....sections.concrete.reinforcement import areas
from . import flexure_tables


def d_for_M(M, bw, μ, fcd):
    return (M / (bw * μ * fcd)) ** 0.5


class RectangularBeam:

    def __init__(self, bw, h, d1, ductility_class='M'):
        self.bw = bw
        self.h = h
        self.d1 = d1
        self.ductility_class = ductility_class

    def set_loads(self, Md, Nd):
        self.Md = Md
        self.Nd = Nd

    def set_materials(self, fck, fyk, acc=0.85, γc=1.5, γs=1.15):
        self.fck = fck
        self.fyk = fyk
        self.acc = acc
        self.γc = γc
        self.γs = γs

    def calc(self):
        if self.ductility_class == 'M':
            μφ = 6.8
        else:
            μφ = 10.7

        self.fcd = self.acc * self.fck / self.γc
        self.fyd = self.fyk / self.γs
        ε = self.fyd / 200000000.

        self.ys1 = self.h / 2 - self.d1
        self.d = self.h - self.d1
        self.Msd = abs(self.Md) - self.Nd * self.ys1
        self.μsd = norm_forces.μ(self.Msd, self.bw, self.d, self.fcd)
        self.ω = flexure_tables.ω_for_μsd(self.μsd)
        self.As = areas.As_for_ω(self.ω, self.bw, self.d, self.fcd, self.fyd, self.Nd)
        self.ρmin = detbeams.ρmin(self.fck/1000., self.fyk/1000.)
        self.ρmax = detbeams.ρmax05(μφ, ε, self.fck * 1.0 / 1.5, self.fyk / 1.15)
        self.Asmin = self.ρmin * self.bw * self.d
        self.Asmaxcr = self.ρmax * self.bw * self.d
        self.Asmaxgen = 0.04 * self.bw * self.d

        if self.μsd >= 0.30:
            self.d2d = self.d1 / self.d
            self.ω1, self.ω2 = flexure_tables.ω1ω2_for_μsd(self.μsd)
            self.ρ1, self.ρ2 = flexure_tables.ρ1ρ2_for_d2d(self.d2d, self.ω1)
            self.As1 = areas.As1_for_ω1ρ1(self.ω1, self.ρ1, self.bw, self.d, self.fcd, self.fyd, self.Nd)
            self.As2 = areas.As2_for_ω2ρ2(self.ω2, self.ρ2, self.bw, self.d, self.fcd, self.fyd)

    def log_list(self):
        _tbl = list()
        _tbl.append(['fcd', self.fcd, 'MPa'])
        _tbl.append(['fyd', self.fyd, 'MPa'])
        _tbl.append(['ys1', self.ys1, 'm'])
        _tbl.append(['d', self.d, 'm'])
        _tbl.append(['Msd', self.Msd, 'kNm'])
        _tbl.append(['μsd', self.μsd, ''])
        _tbl.append(['ω', self.ω, ''])
        _tbl.append(['As', self.As, 'm2'])
        _tbl.append(['Asmin', self.Asmin, 'm2'])
        _tbl.append(['Asmax,κρισ', self.Asmaxcr, 'm2'])
        _tbl.append(['Asmax,γεν', self.Asmaxgen, 'm2'])

        if self.μsd >= 0.30:
            _tbl.append(['Λύση με τοποθέτηση και θλιβόμενου οπλισμού', 0., ''])
            _tbl.append(['d2/d (θεωρώντας d1=d2)', self.d2d, ''])
            _tbl.append(['ω1', self.ω1, ''])
            _tbl.append(['ω2', self.ω2, ''])
            _tbl.append(['ρ1', self.ρ1, ''])
            _tbl.append(['ρ2', self.ρ2, ''])
            _tbl.append(['As1', self.As1, 'm2'])
            _tbl.append(['As2', self.As2, 'm2'])

        return _tbl

    def log_list(self):
        _log = list()
        _log.append({'quantity': 'fcd',
                     'value': self.fcd,
                     'units': 'MPa'})
        _log.append({'quantity': 'fyd',
                     'value': self.fyd,
                     'units': 'MPa'})
        _log.append({'quantity': 'ys1',
                     'value': self.ys1,
                     'units': 'm'})
        _log.append({'quantity': 'd',
                     'value': self.d,
                     'units': 'm'})
        _log.append({'quantity': 'Msd',
                     'value': self.Msd,
                     'units': 'kNm'})
        _log.append({'quantity': 'μsd',
                     'value': self.μsd,
                     'units': ''})
        _log.append({'quantity': 'ω',
                     'value': self.ω,
                     'units': ''})
        _log.append({'quantity': 'As',
                     'value': self.As,
                     'units': 'm2'})
        _log.append({'quantity': 'Asmin',
                     'value': self.Asmin,
                     'units': 'm2'})
        _log.append({'quantity': 'Asmax,κρισ',
                     'value': self.Asmaxcr,
                     'units': 'm2'})
        _log.append({'quantity': 'Asmax,γεν',
                     'value': self.Asmaxgen,
                     'units': 'm2'})

        if self.μsd >= 0.30:
            _log.append({'quantity': 'Λύση με τοποθέτηση και θλιβόμενου οπλισμού',
                         'value': None,
                         'units': ''})
            _log.append({'quantity': 'd2/d (θεωρώντας d1=d2)',
                         'value': self.d2d,
                         'units': ''})
            _log.append({'quantity': 'ω1',
                         'value': self.ω1,
                         'units': ''})
            _log.append({'quantity': 'ω2',
                         'value': self.ω2,
                         'units': ''})
            _log.append({'quantity': 'ρ1',
                         'value': self.ρ1,
                         'units': ''})
            _log.append({'quantity': 'ρ2',
                         'value': self.ρ2,
                         'units': ''})
            _log.append({'quantity': 'As1',
                         'value': self.As1,
                         'units': 'm2'})
            _log.append({'quantity': 'As2',
                         'value': self.As2,
                         'units': 'm2'})


        return _log

    def log_panda(self):
        return results_display_older.log_dataframe(self.log_list())

    def log_markdown_table(self):
        return results_display_older.log_markdown_table(self.log_list())

    def __str__(self):
        return self.log_markdown_table()



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from sympy.solvers import solve
from sympy import Symbol
from dataclasses import dataclass, field
from typing import List
from streng.ppp.sections.concrete.typical_sections.rectangular_rc import RectangularConcreteSection
from streng.codes.greek.kanepe.raw.ch7.rotation import θycalc, θum, αcalc
from streng.codes.greek.kanepe.raw.ch7a.yield_point import ξycalc, Mycalc, ABξφ_conc, ABξφ_steel
from streng.codes.greek.kanepe.raw.ch7c.shear import VRcalc, Vwcalc, VRccalc

from streng.common.io.output import OutputTable, OutputExtended

@dataclass
class RectangularKanepe:
    rcs: RectangularConcreteSection
    Ls: float
    N: float
    bi: List[float] = field(default_factory=list)

    calc_on_init: bool = field(repr=False, default=True)

    results: OutputExtended = field(init=False, repr=False, default_factory=OutputExtended)

    θy: float = field(init=False, repr=False)
    θu_ini: float = field(init=False, repr=False)
    θu: float = field(init=False, repr=False)
    My: float = field(init=False, repr=False)


    results_Mtheta: pd.DataFrame = field(init=False, repr=False)
    figures: List[object] = field(init=False, repr=False, default_factory=list)



    def __post_init__(self):
        if self.calc_on_init == True:
            self.calculate()

    def calculate(self):
        N = self.N

        Ec = self.rcs.materials.Ec
        Es = self.rcs.materials.Es
        fc = self.rcs.materials.fc
        fy = self.rcs.materials.fy
        fyw = self.rcs.materials.fyw
        self.results.outputTables['materials'] = self.rcs.materials.to_OutputTable

        b = self.rcs.b
        h = self.rcs.h

        Ls = self.Ls
        αs = Ls / h

        _out_section_data = []
        _out_section_data.append({'quantity': 'b', 'value': b, 'note': 'm'})
        _out_section_data.append({'quantity': 'h', 'value': h, 'note': 'm'})
        _out_section_data.append({'quantity': 'Ls', 'value': Ls, 'note': 'm (Μήκος διάτμησης)'})
        _out_section_data.append({'quantity': 'αs', 'value': αs, 'note': '(Λόγος διάτμησης)'})


        d1 = self.rcs.reinforcement.d1
        d2 = self.rcs.reinforcement.d2
        d = h - d1
        δ2 = d2 / d
        z = d - d2
        cnom = self.rcs.reinforcement.cnom
        Φw = self.rcs.reinforcement.trans.dia

        ΦL = self.rcs.reinforcement.long1.dia_max

        s = self.rcs.reinforcement.trans.s

        n_b = self.rcs.reinforcement.trans.n               # αριθμός σκελών συνδετήρα κύριας (b) παρειάς
        n_h = self.rcs.reinforcement.side_hoop_legs + 2    # αριθμός σκελών συνδετήρα 2ουσας (h) παρειάς

        _out_section_data.append({'quantity': 'd1', 'value': d1, 'note': 'm'})
        _out_section_data.append({'quantity': 'd2', 'value': d2, 'note': 'm (ή d`)'})
        _out_section_data.append({'quantity': 'd', 'value': d, 'note': 'm'})
        _out_section_data.append({'quantity': 'δ`', 'value': δ2, 'note': '(ή δ2)'})
        _out_section_data.append({'quantity': 'z', 'value': z, 'note': 'm'})
        _out_section_data.append({'quantity': 'cnom', 'value': cnom, 'note': 'm'})
        _out_section_data.append({'quantity': 'Φw', 'value': Φw, 'note': 'm'})
        _out_section_data.append({'quantity': 'ΦL', 'value': ΦL, 'note': 'm'})
        _out_section_data.append({'quantity': 's', 'value': s, 'note': 'm'})


        As = self.rcs.reinforcement.long1.As
        As2 = self.rcs.reinforcement.long2.As
        Asv = self.rcs.reinforcement.longV.As
        Asd = self.rcs.reinforcement.disdia.As
        Asw = self.rcs.reinforcement.trans.As
        Astot = self.rcs.reinforcement.Astot

        _out_section_data.append({'quantity': 'As', 'value': As, 'note': 'm2 Εμβαδό του εφελκυόμενου οπλισμού'})
        _out_section_data.append({'quantity': 'As2', 'value': As2, 'note': 'm2 Εμβαδό του θλιβόμενου οπλισμού'})
        _out_section_data.append({'quantity': 'Asv', 'value': Asv, 'note': 'm2 Εμβαδό του ενδιάμεσου οπλισμού'})
        _out_section_data.append({'quantity': 'Asd', 'value': Asd, 'note': 'm2 Εμβαδό του δισδιαγώνιου οπλισμού'})
        _out_section_data.append({'quantity': 'Asw', 'value': Asw, 'note': 'm2 Εμβαδό του εγκάρσιου οπλισμού'})
        _out_section_data.append({'quantity': 'Astot', 'value': Astot, 'note': 'm2 Συνολικό εμβαδό του διαμήκους οπλισμού, Astot'})


        self.results.outputTables['section'] = OutputTable(data=_out_section_data)

        ρ = self.rcs.ratios.ρ1
        ω = self.rcs.ratios.ω1
        ρ2 = self.rcs.ratios.ρ2
        ω2 = self.rcs.ratios.ω2
        ρv = self.rcs.ratios.ρv
        ωv = self.rcs.ratios.ωv
        ρtot = self.rcs.ratios.ρtot
        ωtot = self.rcs.ratios.ωtot
        ρd = self.rcs.ratios.ρd
        ωd = self.rcs.ratios.ωd
        ρs = self.rcs.ratios.ρw


        self.results.outputTables['ratios'] = self.rcs.ratios.to_OutputTable

        _out_yield_data = []

        α = Es / Ec
        _out_yield_data.append({'quantity': 'α', 'value': α})

        A_steel, B_steel, ξy_steel, φy_steel = ABξφ_steel(ρ, ρ2, ρv, N, b, d, δ2, fy, α, Es)
        _out_yield_data.append({'quantity': 'A_steel', 'value': A_steel})
        _out_yield_data.append({'quantity': 'B_steel', 'value': B_steel})
        _out_yield_data.append({'quantity': 'ξy_steel', 'value': ξy_steel})
        _out_yield_data.append({'quantity': 'φy_steel', 'value': φy_steel, 'note': 'm-1'})

        A_conc, B_conc, ξy_conc, φy_conc = ABξφ_conc(ρ, ρ2, ρv, N, b, d, δ2, α, Ec, fc)
        _out_yield_data.append({'quantity': 'A_conc', 'value': A_conc})
        _out_yield_data.append({'quantity': 'B_conc', 'value': B_conc})
        _out_yield_data.append({'quantity': 'ξy_conc', 'value': ξy_conc})
        _out_yield_data.append({'quantity': 'φy_conc', 'value': φy_conc, 'note': 'm-1'})

        if φy_steel < φy_conc:
            ξy = ξy_steel
            φy = φy_steel
            verdict = 'χάλυβα'
        else:
            ξy = ξy_conc
            φy = φy_conc
            verdict = 'μη-γραμμικότητας των παραμορφώσεων του θλιβόµενου σκυροδέματος'

        My = Mycalc(b, d, φy, Ec, ξy, δ2, ρ, ρ2, ρv, Es)

        xy = ξy * d

        _out_yield_data.append({'quantity': f'Διαρροή λόγω', 'value': None, 'note': verdict})
        _out_yield_data.append({'quantity': 'ξy', 'value': ξy})
        _out_yield_data.append({'quantity': 'xy', 'value': xy, 'note': 'm (Το ύψος της θλιβόμενης ζώνης στη διαρροή)'})
        _out_yield_data.append({'quantity': 'φy', 'value': φy, 'note': 'm-1'})
        _out_yield_data.append({'quantity': 'My', 'value': My, 'note': 'kNm'})


        VRc = VRccalc(ρtot, b, d, fc / 1000, N, b * h)

        VMu = My / Ls
        if VRc < VMu:
            αv = 1.
        else:
            αv = 0.

        θy = θycalc(φy, Ls, αv, z, h, ΦL, fy/1000., fc/1000.)

        _out_yield_data.append({'quantity': 'VRc', 'value': VRc, 'note': 'kN'})
        _out_yield_data.append({'quantity': 'αv', 'value': αv})
        _out_yield_data.append({'quantity': 'VMu', 'value': VMu, 'note': 'kN'})
        _out_yield_data.append({'quantity': 'θy', 'value': θy})

        self.results.outputTables['yield'] = OutputTable(data=_out_yield_data)


        _out_ultimate_point_data = []

        ν = N / (b*h*fc)
        _out_ultimate_point_data.append({'quantity': 'ν', 'value': ν})

        bo = b - 2 * (cnom + 0.5*Φw)
        ho = h - 2 * (cnom + 0.5*Φw)
        _out_ultimate_point_data.append({'quantity': 'bo', 'value': bo})
        _out_ultimate_point_data.append({'quantity': 'ho', 'value': ho})

        if ho/bo > 2.0:
            _out_ultimate_point_data.append({'quantity': 'hο/bο', 'value': ho / bo,
                                             'note': 'hο/bο>2.0 Χρησιμοποιείται, ως ho, το ύψος της θλιβόµενης ζώνης εντός του περισφιγμένου πυρήνα'})
            ho = xy - (cnom + 0.5*Φw)
            _out_ultimate_point_data.append({'quantity': 'ho', 'value': ho, 'note': 'Τελική τιμή'})


        if len(self.bi) == 0.0:
            # self.bi = np.array([bo, ho, bo, ho])  # Για 2τμητους συνδετήρες

            spaces_b = int(round(n_b - 1, 0))
            spaces_h = int(round(n_h - 1, 0))
            space_length_b = bo / spaces_b
            space_length_h = ho / spaces_h
            bi = []
            for i in range(2 * spaces_h): bi.append(space_length_h)
            for i in range(2 * spaces_b): bi.append(space_length_b)
            self.bi = np.array(bi)


        Σbi2 = sum(self.bi**2)
        απερ = αcalc(s, bo, ho, Σbi2)
        θu_ini = θum(ν, ωtot, ω2, αs, απερ, ρs, ρd, fc / 1000, fyw / 1000)
        μθ_ini = θu_ini / θy

        Vw = Vwcalc(ρs, b, z, fyw)


        _out_ultimate_point_data.append({'quantity': 'Σbi2', 'value': Σbi2, 'note': 'm2'})
        _out_ultimate_point_data.append({'quantity': 'απερ', 'value': απερ, 'note': 'Συντελεστής αποδοτικότητας περίσφιξης'})
        _out_ultimate_point_data.append({'quantity': 'θu_ini', 'value': θu_ini, 'note': 'Αρχική τιμή στροφής αστοχίας (χωρίς τον έλεγχο για τις τέμνουσες)'})
        _out_ultimate_point_data.append({'quantity': 'μθ_ini', 'value': μθ_ini, 'note': 'Αρχική τιμή πλαστιμότητας στροφών (χωρίς τον έλεγχο για τις τέμνουσες)'})
        _out_ultimate_point_data.append({'quantity': 'Vw', 'value': Vw, 'note': 'kN'})


        υ = Symbol('υ')
        μθplsol = float(solve((((h - xy) / (2 * Ls)) * min(N / 1000, 0.55 * b * d * fc / 1000) + (1 - 0.05 * υ) * (
                    0.16 * max(0.5, 100 * ρtot) * (1 - 0.16 * min(5, αs)) * (
                        (fc / 1000) ** 0.5) * b * d + Vw / 1000) * 1000 - My / Ls), υ)[0])
        θsol = θy * (μθplsol + 1)

        _out_ultimate_point_data.append({'quantity': 'μθplsol', 'value': μθplsol, 'note': ''})
        _out_ultimate_point_data.append({'quantity': 'θsol', 'value': θsol, 'note': ''})


        VR0 = VRcalc(h, xy, Ls, N / 1000, b * d, fc / 1000, 0., ρtot, αs, Vw / 1000) * 1000 # yVRs[0]
        My_ini = My
        θy_ini = θy
        if VR0 < My / Ls:
            θu = 1.4 * θy
            θy = θy * VR0 * Ls / My
            My = VR0 * Ls
            _out_ultimate_point_data.append({'quantity': 'Πρόωρη αστοχία', 'note': 'από τέμνουσα πριν τη διαρροή'})

        elif θsol < θu_ini:
            θu = θsol
            _out_ultimate_point_data.append({'quantity': 'Πρόωρη αστοχία','note': 'από τέμνουσα πριν την εξάντληση της θu'})

        else:
            θu = θu_ini

        μθ = θu/θy
        θpl = θu - θy
        _out_ultimate_point_data.append({'quantity': 'θu', 'value': θu, 'note': 'Τελική τιμή'})
        _out_ultimate_point_data.append({'quantity': 'μθ', 'value': μθ, 'note': 'Τελική τιμή'})
        _out_ultimate_point_data.append({'quantity': 'θpl', 'value': θpl, 'note': 'Λαμβάνεται ως θpl = θu - θy'})


        self.results.outputTables['ultimate'] = OutputTable(data=_out_ultimate_point_data)


        self.θy = θy
        self.My_ini = My_ini
        self.θy_ini = θy_ini
        self.θu_ini = θu_ini
        self.My = My
        self.θu = θu
        self.θpl = θpl
        self.Mu = My  # θεωρώ για την ώρα μηδενική κράτυνση



        # da = {'M': [0, My, My, 0.2 * My, 0.2 * My], 'θ': [0, 0, θpl, θpl, 5 * θpl]}
        # self.results_Mtheta = pd.DataFrame(data=da, index=['A', 'B', 'C', 'D', 'E'])
        # print(df.round({'M': 2, 'θ': 4}))


    def get_etabs9_7_2_M3(self, negative = False):
        θy_ini = self.θy_ini
        θu_ini = self.θu_ini
        My = self.My_ini
        Mu = self.My_ini


        if negative == True:
            return(f'TYPE "M3"  -E "-{5.*self.θpl:.4f} -{0.2*self.My:.2f}"  -D "-{1.5*self.θpl:.4f} -{0.2*self.My:.2f}"  -C "-{self.θpl:.4f} -{self.My:.2f}"  -B "0 -{self.My:.2f}"')
        else:
            return (f'TYPE "M3"  B "0 {self.My:.2f}"  C "{self.θpl:.4f} {self.My:.2f}"  D "{1.5*self.θpl:.4f} {0.2*self.My:.2f}"  E "{5.*self.θpl:.4f} {0.2*self.My:.2f}"')

    def print_output_tables(self):
        for key, value in self.results.outputTables.items():
            print(key)
            print(value)
            print('')


    @property
    def figure_initial(self):
        θy_ini = self.θy_ini
        θu_ini = self.θu_ini
        My = self.My_ini
        Mu = self.My_ini

        xMθs = [0., θy_ini, θu_ini]
        yMθs = [0., My, Mu]

        f, ax = plt.subplots(figsize=(8, 5))
        ax.plot(xMθs, yMθs, lw=2)
        ax.axis([0, 1.2 * θu_ini, 0, 1.2 * Mu])
        ax.set_title('Αρχικό διάγραμμα Μ-θ (χωρίς τον έλεγχο για τις τέμνουσες)')
        ax.set_ylabel('M (kN)')
        ax.set_xlabel('θ')
        fig = (f, ax)

        return fig


    @property
    def figure_final(self):
        θy = self.θy
        θu = self.θu
        My = self.My
        Mu = self.My

        xMθs = [0., θy, θu]
        yMθs = [0., My, Mu]

        f, ax = plt.subplots(figsize=(8, 5))
        ax.plot(xMθs, yMθs, lw=2)
        ax.axis([0, 1.2 * θu, 0, 1.2 * Mu])
        ax.set_title('Τελικό διάγραμμα Μ-θ')
        ax.set_ylabel('M (kN)')
        ax.set_xlabel('θ')
        fig = (f, ax)

        return fig

    @property
    def figure_shear_strength(self):
        xs = np.linspace(0., 1.5 * self.θu_ini, 100)
        yVRs = list()
        yMs = list()

        xy = self.results.outputTables['yield'].retrieve(search_field='quantity',
                                                         search_value='xy',
                                                         find_field='value')

        αs = self.results.outputTables['section'].retrieve(search_field='quantity',
                                                           search_value='αs',
                                                           find_field='value')

        Vw = self.results.outputTables['ultimate'].retrieve(search_field='quantity',
                                                            search_value='Vw',
                                                            find_field='value')

        for x in xs:
            μθpl = max(0, (x / self.θy) - 1)
            VRi = VRcalc(self.rcs.h, xy, self.Ls, self.N / 1000, self.rcs.b * self.rcs.d, self.rcs.materials.fc / 1000, μθpl, self.rcs.ratios.ρtot, αs, Vw / 1000) * 1000
            MVRi = VRi * self.Ls

            yVRs.append(VRi)
            yMs.append(MVRi)

        xsy = [self.θy, self.θy]
        ysy = [0, 1.2 * max(yVRs)]

        xsu = [self.θu_ini, self.θu_ini]
        ysu = [0, 1.2 * max(yVRs)]

        f, ax = plt.subplots(figsize=(8, 5))
        ax.plot(xs, yVRs, label="Τέμνουσα αντοχής VR", lw=2)
        ax.plot(xsy, ysy, label="Διαρροή", linestyle='--')
        ax.plot(xsu, ysu, label="Καμπτική αστοχία", linestyle='--')
        ax.axis([0, 1.5 * self.θu_ini, 0, 1.2 * max(yVRs)])
        ax.set_title('Μείωση της τέμνουσας αντοχής με την ανελαστική παραμόρφωση')
        ax.set_ylabel('VR (kN)')
        ax.set_xlabel('θ')
        ax.legend()
        fig = (f, ax)

        return fig

    @property
    def figure_shear_check(self):
        xs = np.linspace(0., 1.5 * self.θu_ini, 100)
        yVRs = list()
        yMs = list()

        xy = self.results.outputTables['yield'].retrieve(search_field='quantity',
                                                         search_value='xy',
                                                         find_field='value')

        αs = self.results.outputTables['section'].retrieve(search_field='quantity',
                                                           search_value='αs',
                                                           find_field='value')

        Vw = self.results.outputTables['ultimate'].retrieve(search_field='quantity',
                                                            search_value='Vw',
                                                            find_field='value')

        for x in xs:
            μθpl = max(0, (x / self.θy) - 1)
            VRi = VRcalc(self.rcs.h, xy, self.Ls, self.N / 1000, self.rcs.b * self.rcs.d, self.rcs.materials.fc / 1000, μθpl, self.rcs.ratios.ρtot, αs, Vw / 1000) * 1000
            MVRi = VRi * self.Ls

            yVRs.append(VRi)
            yMs.append(MVRi)

        ymax = max(max(yMs), self.My_ini)

        xsy = [self.θy_ini, self.θy_ini]
        ysy = [0, 1.2 * ymax]

        xsu = [self.θu_ini, self.θu_ini]
        ysu = [0, 1.2 * ymax]

        xMθs = [0, self.θy_ini, self.θu_ini]
        yMθs = [0, self.My_ini, self.My_ini]


        f, ax = plt.subplots(figsize=(8, 5))
        ax.plot(xs, yMs, label="Ροπή τέμνουσας αντοχής (VR*Ls)", lw=2)
        ax.plot(xsy, ysy, label="Διαρροή", linestyle='--')
        ax.plot(xsu, ysu, label="Αστοχία", linestyle='--')
        ax.axis([0, 1.5 * self.θu_ini, 0, 1.2 * ymax])
        ax.set_title('Έλεγχος πρόωρης αστοχίας από διάτμηση')
        ax.set_ylabel('M (kN)')
        ax.set_xlabel('θ')
        ax.plot(xMθs, yMθs, label="Αρχικό διάγραμμα Μ-θ")
        ax.legend()
        fig = (f, ax)

        return fig




# @dataclass
# class Rectangular:
#     b: float # το πλάτος της θλιβόμενης ζώνης[m]
#     h: float # το ύψος της διατομής[m]
#
#     reinforcement: Dict[str, object] = field(default_factory=dict)
#     materials: Dict[str, float] = field(default_factory=dict)
#
#     # Οπλισμός
#     N: float            # Αξονικό φορτίο [kN]
#     # Μήκος διάτμησης
#     Ls: float        # [m]
#     # Αποστάσεις bi περίσφιξης [m]
#     bi = np.array([0.0, 0.0])
#
#
#     cnom: float  # [m]
#     ## Διαμήκης
#     ### Εφελκυόμενος
#     nL: float  # Ο αριθτμός των εφελκυόμενων ράβδων
#     ΦL: float  # Η διάμετρος των εφελκυόμενων ράβδων [m]
#     ### Θλιβόμενος
#     nL2: float  # Ο αριθτμός των θλιβόμενων ράβδων
#     ΦL2: float  # Η διάμετρος των θλιβόμενων ράβδων [m]
#     ### Ενδιάμεσος
#     nLv: float # Ο αριθτμός των ενδιάμεσων ράβδων
#     ΦLv: float  # Η διάμετρος των ενδιάμεσων ράβδων [m]
#     ## Δισδιαγώνιος
#     nd: float # Ο αριθτμός των δισδιαγώνιων ράβδων
#     Φd: float  # Η διάμετρος των δισδιαγώνιων ράβδων [m]
#     ## Εγκάρσιος
#     nw: float  # Ο αριθμός των "τμήσεων" των συνδετήρων (πχ. 2 για 2τμητους)
#     Φw: float  # Η διάμετρος των συνδετήρων [m]
#     s: float  # Η απόσταση μεταξύ των συνδετήρων [m]
#
#
#     fc: float  # η θλιπτική αντοχή του σκυροδέματος[kPa]
#     Ec: float  # το μέτρο ελαστικότητας του σκυροδέματος [kPa]
#     fy: float  # το όριο διαρροής του χάλυβα [kPa]
#     Es: float  # το μέτρο ελαστικότητας του χάλυβα [kPa]
#     fyw: float # το όριο διαρροής του χάλυβα των συνδετήρων [kPa]






import math
from tabulate import tabulate

def ξycalc(α, A, B, makelatex=False):
    """
        Το ύψος της θλιβόµενης ζώνης στη διαρροή, ξy, ανηγµένο στο στατικό ύψος d, είναι:

        .. math::
            ξ_y=(α^2A^2+2αB)^{0.5}-αA

    Args:
        α (float): Ο λόγος των μέτρων ελαστικότητας :math:`α = E_s/E_c`
        A (float): Ο συντελεστής A που υπολογίζονται για διαρροή λόγω χάλυβα ή μη-γραμμικότητα σκυροδέματος
        B (float): Ο συντελεστής B που υπολογίζονται για διαρροή λόγω χάλυβα ή μη-γραμμικότητα σκυροδέματος
        makelatex (bool): True για να δημιουργηθεί το latex string

    Returns:
        tuple: A tuple, with [value, latexstring]
    """
    value = math.pow((math.pow(α, 2) * math.pow(A, 2) + 2 * α * B), 0.5) - α * A
    if makelatex:
        text = r"ξ_y=(α^2A^2+2αB)^{0.5}-αA" \
               + f'=({α:.3f}^2\cdot {A:.5f}^2+2\cdot {α:.3f}\cdot {B:.5f})^{{0.5}}-{α:.3f}\cdot {A:.5f}={value:.4f}'
    else:
        text = ''

    return value, text


def yield_props(ρ1, ρ2, ρv, N, b, d, δtonos, fc, Ec, fy, Es, makelog=False, makelatex=False):
    logtext = ''
    latext = ''

    α = Es/Ec
    Asteel = ρ1 + ρ2 + ρv + N / (b * d * fy)
    Bsteel = ρ1 + ρ2 * δtonos + 0.5 * ρv * (1 + δtonos) + N / (b * d * fy)
    ξysteel = ξycalc(α, Asteel, Bsteel, makelatex)
    φysteel = fy / (Es * (1.0 - ξysteel[0]) * d)
    Aconc = ρ1 + ρ2 + ρv - N / (1.8 * α * b * d * fc)
    Bconc = ρ1 + ρ2 * δtonos + 0.5 * ρv * (1 + δtonos)
    ξyconc = ξycalc(α, Aconc, Bconc, makelatex)
    φyconc = 1.8 * fc / (Ec * ξyconc[0] * d)

    if φysteel<φyconc:
        φy = φysteel
        ξy = ξysteel[0]
    else:
        φy = φyconc
        ξy = ξyconc[0]

    if makelog:
        headers = ['', 'value', 'units']
        table = [['α', α],
                 ['Asteel', Asteel],
                 ['Bsteel', Bsteel],
                 ['ξysteel', ξysteel[0], 'm'],
                 ['φysteel', φysteel, 'm-1'],
                 ['Aconc', Aconc],
                 ['Bconc', Bconc],
                 ['ξyconc', ξyconc[0]],
                 ['φyconc', φyconc, 'm-1'],
                 ['φy', φy, 'm-1'],
                 ['ξy', ξy, 'm']]
        logtext = tabulate(table, headers, tablefmt="pipe", floatfmt=".3E")


    return α, Asteel, Bsteel, ξysteel[0], φysteel, Aconc, Bconc, ξyconc[0], φyconc, φy, ξy, logtext, latext

def A_steel(ρ1, ρ2, ρv, N, b, d, fy, makelatex=False):
    """
        Συντελεστής A για διαρροή του οπλισμού. 
        Δίνεται από τη σχέση:

        .. math::
            A=ρ+ρ'+ρ_v+\dfrac{N}{bdf_y}     
            
    Args:
        ρ1 (float): (ρ) το ποσοστό του εφελκυόµενου οπλισμού
        ρ2 (float): (ρ') το ποσοστό του θλιβόμενου οπλισμού
        ρv (float): το ποσοστό του μεταξύ τους κατανεμημένου οπλισμού
        N (float): το αξονικό φορτίο (θετικό σε θλίψη) [kN]
        b (float): το πλάτος της θλιβόµενης ζώνης [m]
        d (float): το στατικό ύψος  [m]
        fy (float): το όριο διαρροής του χάλυβα  [kPa]
        makelatex (bool): True για να δημιουργηθεί το latex string

    Returns:
        tuple: A tuple, with [value, latexstring]
    """
    value = ρ1 + ρ2 + ρv + N / (b * d * fy)
    if makelatex:
        # text = r"A=ρ+ρ'+ρ_v+\dfrac{{N}}{{bdf_y}}=" \
        #    + '{0:.5f}+{1:.5f}+{2:.5f}+'.format(ρ1, ρ2, ρv) \
        #    + r'\dfrac{{{0:.2f}kN}}{{{1:.2f}m\cdot{2:.2f}m\cdot{3:.1f}kPa}}={4:.5f}'.format(N, b, d, fy, value)
        text = r"A=ρ+ρ'+ρ_v+\dfrac{{N}}{{bdf_y}}=" \
           + f'{ρ1:.5f}+{ρ2:.5f}+{ρv:.5f}+' \
           + f'\dfrac{{{N:.2f}kN}}{{{b:.2f}m\cdot{d:.2f}m\cdot{fy:.1f}kPa}}={value:.5f}'
    else:
        text = ''

    return value, text


def B_steel(ρ1, ρ2, ρv, N, b, d, δtonos, fy, makelatex=False):
    """
        Συντελεστής B για διαρροή του οπλισμού. 
        Δίνεται από τη σχέση:
    
        .. math::
            B=ρ+ρ'δ'+0.5ρ_v(1+δ')+\dfrac{N}{bdf_y}
        
    Args:
        ρ1 (float): (ρ) το ποσοστό του εφελκυόµενου οπλισμού
        ρ2 (float): (ρ') το ποσοστό του θλιβόμενου οπλισμού
        ρv (float): το ποσοστό του μεταξύ τους κατανεμημένου οπλισμού
        N (float): το αξονικό φορτίο (θετικό σε θλίψη) [kN]
        b (float): το πλάτος της θλιβόµενης ζώνης [m]
        d (float): το στατικό ύψος [m]
        δtonos (float): :math:`δ' = d'/d` όπου d' η απόσταση από το κέντρο του θλιβόµενου οπλισµού µέχρι την ακραία θλιβόµενη ίνα σκυροδέµατος
        fy (float): το όριο διαρροής του χάλυβα  [kPa]
        makelatex (bool): True για να δημιουργηθεί το latex string

    Returns:
        tuple: A tuple, with [value, latexstring]
    """

    value = ρ1 + ρ2 * δtonos + 0.5 * ρv * (1 + δtonos) + N / (b * d * fy)
    if makelatex:
        # text = r"B=ρ+ρ'δ'+0.5ρ_v(1+δ')+\dfrac{N}{bdf_y}=" \
        #    + r'{0:.5f}+{1:.5f}\cdot{{{3:.3f}}}+0.5\cdot{2:.5f}\cdot(1+{{{3:.3f}}})+'.format(ρ1, ρ2, ρv, δtonos) \
        #    + r'\dfrac{{{0:.2f}kN}}{{{1:.2f}m\cdot{2:.2f}m\cdot{3:.1f}kPa}}={4:.5f}'.format(N, b, d, fy, value)
        text = r'\begin{aligned}' \
           + r"B &=ρ+ρ'δ'+0.5ρ_v(1+δ')+\dfrac{N}{bdf_y}=" + r'\\' \
           + f'&={ρ1:.5f}+{ρ2:.5f}\cdot{{{δtonos:.3f}}}+0.5\cdot{ρv:.5f}\cdot(1+{{{δtonos:.3f}}})+'  \
           + f'\dfrac{{{N:.2f}kN}}{{{b:.2f}m\cdot{d:.2f}m\cdot{fy:.1f}kPa}}={value:.5f}' \
           + r'\end{aligned}'
    else:
        text = ''


    return value, text


def φy_steel(fy, Es, ξy, d, makelatex=False):
    """
        Καμπυλότητα διαρροής για διαρροή του οπλισμού.
        Δίνεται από τη σχέση:
    
        .. math::        
            φ_y=\dfrac{f_y}{E_s(1-ξ_y)d}
        
    Args:
        fy (float): το όριο διαρροής του χάλυβα  [kPa]
        Es (float): το μέτρο ελαστικότητας του χάλυβα  [kPa]
        ξy (float): Το ύψος της θλιβόµενης ζώνης στη διαρροή, ανηγµένο στο στατικό ύψος d [m]
        d (float): το στατικό ύψος [m]
        makelatex (bool): True για να δημιουργηθεί το latex string

    Returns:
        tuple: A tuple, with [value, latexstring]
    """

    value = fy / (Es * (1.0 - ξy) * d)
    if makelatex:
        # text = r"φ_y=\dfrac{f_y}{E_s(1-ξ_y)d}=" \
        #    + r'\dfrac{{{0:.1f}}}{{{1:.1f}(1-{2:.4f}){3:.3f}}}={4:.5f}'.format(fy, Es, ξy, d, value)
        text = r"φ_y=\dfrac{f_y}{E_s(1-ξ_y)d}=" \
           + f'\dfrac{{{fy:.1f}}}{{{Es:.1f}\cdot(1-{ξy:.4f})\cdot{d:.3f}}}={value:.5f}'
    else:
        text = ''

    return value, text


def A_conc(ρ1, ρ2, ρv, N, b, d, α, fc, makelatex=False):
    """
        Συντελεστής A για µή-γραµµικότητα των παραµορφώσεων του θλιβόµενου σκυροδέµατος
        Δίνεται από τη σχέση (χρησιμοποιείται η προσέγγιση):
    
        .. math::
            A=ρ+ρ'+ρ_v-\dfrac{N}{ε_c E_s bd}≈ρ+ρ'+ρ_v-\dfrac{N}{1.8αbdf_c}           
        
    Args:
        ρ1 (float): (ρ) το ποσοστό του εφελκυόµενου οπλισμού
        ρ2 (float): (ρ') το ποσοστό του θλιβόμενου οπλισμού
        ρv (float): το ποσοστό του μεταξύ τους κατανεμημένου οπλισμού
        N (float): το αξονικό φορτίο (θετικό σε θλίψη) [kN]
        b (float): το πλάτος της θλιβόµενης ζώνης [m]
        d (float): το στατικό ύψος  [m]
        α (float): Ο λόγος των μέτρων ελαστικότητας :math:`α = E_s/E_c`
        fc (float): η θλιπτική αντοχή του σκυροδέματος  [kPa]
        makelatex (bool): True για να δημιουργηθεί το latex string

    Returns:
        tuple: A tuple, with [value, latexstring]
    """
    value = ρ1 + ρ2 + ρv - N / (1.8 * α * b * d * fc)
    if makelatex:
        text = r"A=ρ+ρ'+ρ_v-\dfrac{N}{1.8αbdf_c}=" \
           + f'{ρ1:.5f}+{ρ2:.5f}+{ρv:.5f}-' \
           + f'\dfrac{{{N:.2f}kN}}{{1.8\cdot{α:.3f}\cdot{b:.2f}m\cdot{d:.2f}m\cdot{fc:.1f}kPa}}={value:.5f}'
    else:
        text = ''

    return value, text


def B_conc(ρ1, ρ2, ρv, b, d, δtonos, makelatex=False):
    """
        Συντελεστής B για µή-γραµµικότητα των παραµορφώσεων του θλιβόµενου σκυροδέµατος
         Δίνεται από τη σχέση:
    
        .. math::
            B=ρ+ρ'δ'+0.5ρ_v(1+δ')
                   
    Args:
        ρ1 (float): (ρ) το ποσοστό του εφελκυόµενου οπλισμού
        ρ2 (float): (ρ') το ποσοστό του θλιβόμενου οπλισμού
        ρv (float): το ποσοστό του μεταξύ τους κατανεμημένου οπλισμού
        b (float): το πλάτος της θλιβόµενης ζώνης [m]
        d (float): το στατικό ύψος  [m]
        δtonos (float): :math:`δ' = d'/d` όπου d' η απόσταση από το κέντρο του θλιβόµενου οπλισµού µέχρι την ακραία θλιβόµενη ίνα σκυροδέµατος
        makelatex (bool): True για να δημιουργηθεί το latex string

    Returns:
        tuple: A tuple, with [value, latexstring]

    """
    value =  ρ1 + ρ2 * δtonos + 0.5 * ρv * (1 + δtonos)
    if makelatex:
        text = r"B=ρ+ρ'δ'+0.5ρ_v(1+δ')=" \
           + f'{ρ1:.5f}+{ρ2:.5f}\cdot{{{δtonos:.3f}}}+0.5\cdot{ρv:.5f}\cdot(1+{{{δtonos:.3f}}})={value:.5f}'
    else:
        text = ''

    return value, text


def φy_conc(fc, Ec, ξy, d, makelatex=False):
    """
        Καμπυλότητα διαρροής για για µή-γραµµικότητα των παραµορφώσεων του θλιβόµενου σκυροδέµατος
        Δίνεται από τη σχέση (χρησιμοποιείται η προσέγγιση):
        
        .. math::
            φ_y=\dfrac{ε_c}{ξ_y d}≈\dfrac{1.8 f_y}{E_s ξ_y d}
        
    Args:
        fc (float): η θλιπτική αντοχή του σκυροδέματος  [kPa]
        Ec (float): το μέτρο ελαστικότητας του σκυροδέματος  [kPa]
        ξy (float): Το ύψος της θλιβόµενης ζώνης στη διαρροή, ανηγµένο στο στατικό ύψος d [m]
        d (float): το στατικό ύψος [m]
        makelatex (bool): True για να δημιουργηθεί το latex string

    Returns:
        tuple: A tuple, with [value, latexstring]
    """

    value = 1.8 * fc / (Ec * ξy * d)
    if makelatex:
        text = r"φ_y=\dfrac{1.8 f_y}{E_s ξ_y d}=" \
               + f'\dfrac{{1.8\cdot{fc:.1f}}}{{{Ec:.1f}\cdot{ξy:.4f}\cdot{d:.3f}}}={value:.5f}'
    else:
        text = ''

    return value, text


def My(b, d, φy, Ec, ξy, δtonos, ρ1, ρ2, ρv, Es, makelatex=False):
    """
        Με δεδοµένη την καµπυλότητα στη διαρροή, η αντίστοιχη ροπή Μy προκύπτει ως:
        
        .. math::
            \dfrac{M_y}{bd^3}=φ_y\Bigg\{E_c\dfrac{ξ_y^2}{2}\Big(0.5(1+δ')-\dfrac{ξ_y}{3}\Big) + \Big[ (1-ξ_y)ρ+(ξ_y-δ')ρ'+\dfrac{ρ_v}{6}(1-δ') \Big]\cdot(1-δ')\dfrac{E_s}{2} \Bigg\}

    Args:
        b (float): το πλάτος της θλιβόµενης ζώνης [m]
        d (float): το στατικό ύψος [m]
        φy (float): Καμπυλότητα διαρροής [m-1]
        Ec (float): το μέτρο ελαστικότητας του σκυροδέματος  [kPa]
        ξy (float): Το ύψος της θλιβόµενης ζώνης στη διαρροή ανηγµένο στο στατικό ύψος d
        δtonos (float): :math:`δ' = d'/d` όπου d' η απόσταση από το κέντρο του θλιβόµενου οπλισµού µέχρι την ακραία θλιβόµενη ίνα σκυροδέµατος
        ρ1 (float): (ρ) το ποσοστό του εφελκυόµενου οπλισμού
        ρ2 (float): (ρ') το ποσοστό του θλιβόμενου οπλισμού
        ρv (float): το ποσοστό του μεταξύ τους κατανεμημένου οπλισμού
        Es (float): το μέτρο ελαστικότητας του χάλυβα  [kPa]
        makelatex (bool): True για να δημιουργηθεί το latex string

    Returns:
        tuple: A tuple, with [value, latexstring]

    """
    value = b * math.pow(d, 3) * φy * (0.5 * Ec * math.pow(ξy, 2) * (0.5 * (1 + δtonos) - ξy / 3) + ((1 - ξy) * ρ1 +
        (ξy - δtonos) * ρ2 + ρv * (1 - δtonos) / 6) * (1 - δtonos) * Es / 2)
    if makelatex:
        text = r'\begin{aligned}' \
               r"M_y &=φ_ybd^3\Bigg\{E_c\dfrac{ξ_y^2}{2}\Big(0.5(1+δ')-\dfrac{ξ_y}{3}\Big) + \Big[ (1-ξ_y)ρ+(ξ_y-δ')ρ'+\dfrac{ρ_v}{6}(1-δ') \Big]\cdot(1-δ')\dfrac{E_s}{2} \Bigg\}=" + r'\\' \
               + f'&={φy:.4f}\cdot {b:.3f} \cdot {d:.3f}^3 \cdot \Bigg\{Ec:.1f}\cdot\dfrac{{{ξy:.4f}^2}}{{2}}\Big(0.5\cdot(1+{δtonos:.3f}) - \dfrac{{{ξy:.4f}}}{{3}}\Big) + ' + r'\\' \
               + f'&+ \Big[ (1-{ξy:.4f})\cdot{ρ1:.5f}+({ξy:.4f}-{δtonos:.3f})\cdot{ρ2:.5f}+\dfrac{{{ρv:.5f}}}{6}(1-{δtonos:.3f}) \Big] \cdot(1-{δtonos:.3f})\dfrac{{{Es:.1f}}}{{2}} \Bigg\}}= ' + r'\\' \
               + f'&={value:.3f}kNm' \
               + r'\end{aligned}'
    else:
        text = ''

    return value, text


# text = r'\begin{aligned}' \
#        + r"B &=ρ+ρ'δ'+0.5ρ_v(1+δ')+\dfrac{N}{bdf_y}=" + r'\\' \
#        + f'&={ρ1:.5f}+{ρ2:.5f}\cdot{{{δtonos:.3f}}}+0.5\cdot{ρv:.5f}\cdot(1+{{{δtonos:.3f}}})+' \
#        + f'\dfrac{{{N:.2f}kN}}{{{b:.2f}m\cdot{d:.2f}m\cdot{fy:.1f}kPa}}={value:.5f}' \
#        + r'\end{aligned}'


def ξycalc(α, A, B):
    return (α ** 2 * A ** 2 + 2 * α * B) ** 0.5 - α * A


def Mycalc(b, d, φy, Ec, ξy, δ2, ρ1, ρ2, ρv, Es):
    return b * d ** 3 * φy * (0.5 * Ec * ξy ** 2 * (0.5 * (1 + δ2) - ξy / 3) + (
                (1 - ξy) * ρ1 + (ξy - δ2) * ρ2 + ρv * (1 - δ2) / 6) * (1 - δ2) * Es / 2)

def ABξφ_steel(ρ, ρ2, ρv, N, b, d, δ2, fy, α, Es):
    A = ρ + ρ2 + ρv + N / (b * d * fy)
    B = ρ + ρ2 * δ2 + 0.5 * ρv * (1 + δ2) + N / (b * d * fy)
    ξy = ξycalc(α, A, B)
    φy = fy / (Es * (1 - ξy) * d)
    return A, B, ξy, φy

def ABξφ_conc(ρ, ρ2, ρv, N, b, d, δ2, α, Ec, fc):
    A = ρ + ρ2 + ρv - N / (1.8 * α * b * d * fc)
    B = ρ + ρ2 * δ2 + 0.5 * ρv * (1 + δ2)
    ξy = ξycalc(α, A, B)
    φy = 1.8 * fc / (Ec * ξy * d)
    return A, B, ξy, φy


import pandas as pd
import xlwings as xw
import numpy as np

from ....codes.eurocodes.ec2.raw.ch3.concrete import strength as conc_strength

@xw.func
def xwf_codes_eurocodes_ec2_ray_ch3_concrete_strength_fcm(fck):
    """fcm"""
    return conc_strength.fcm(fck)






from dataclasses import dataclass
from streng.common.io.output import OutputTable, OutputString


@dataclass
class SectionMaterials:
    fc: float
    Ec: float
    fy: float
    fyw: float
    Es: float

    @property
    def to_OutputTable(self):
        data = []
        data.append({'quantity': 'fc', 'value': self.fc})
        data.append({'quantity': 'Ec', 'value': self.Ec})
        data.append({'quantity': 'fy', 'value': self.fy})
        data.append({'quantity': 'fyw', 'value': self.fyw})
        data.append({'quantity': 'Es', 'value': self.Es})
        return OutputTable(data=data)


@dataclass
class ReinforcementRatios:
    ρ1: float
    ρ2: float
    ρv: float
    ρd: float
    ρtot: float
    ρw: float
    ω1: float
    ω2: float
    ωv: float
    ωd: float
    ωtot: float

    @property
    def to_string(self):
        data = []
        data.append(f'Ποσοστό του εφελκυόμενου οπλισμού: ρ = {self.ρ1:.5f} = {1000*self.ρ1:.2f}‰')
        data.append(f'Μηχανικό ποσοστό του εφελκυόμενου οπλισμού: ω = {self.ω1:.3f}')
        data.append(f'Ποσοστό του θλιβόμενου οπλισμού: ρ` = ρ2 = {self.ρ2:.5f} = {1000*self.ρ2:.2f}‰')
        data.append(f'Μηχανικό ποσοστό του θλιβόμενου οπλισμού: ω` = ω2 = {self.ω2:.3f}')
        data.append(f'Ποσοστό του ενδιάμεσου οπλισμού: ρv = {self.ρv:.5f} = {1000*self.ρv:.2f}‰')
        data.append(f'Μηχανικό ποσοστό του ενδιάμεσου οπλισμού: ωv = {self.ωv:.3f}')
        data.append(f'Συνολικό ποσοστό του διαμήκους οπλισμού: ρtot = {self.ρtot:.5f} = {1000*self.ρtot:.2f}‰')
        data.append(f'Συνολικό μηχανικό ποσοστό του διαμήκους οπλισμού: ωtot = {self.ωtot:.3f}')
        data.append(f'Ποσοστό του δισδιαγώνιου οπλισμού: ρd = {self.ρd:.5f} = {1000*self.ρd:.2f}‰')
        data.append(f'Μηχανικό ποσοστό του δισδιαγώνιου οπλισμού: ωd = {self.ωd:.3f}')
        data.append(f'Ποσοστό του εγκάρσιου οπλισμού (σχέση ΚΑΝ.ΕΠΕ.): ρs = {self.ρw:.5f} = {1000*self.ρw:.2f}‰')
        return OutputString(data=data)

    @property
    def to_OutputTable(self):
        data = []
        data.append({'quantity': 'ρ1', 'value': self.ρ1, 'notes': 'Ποσοστό του εφελκυόμενου οπλισμού'})
        data.append({'quantity': 'ρ2', 'value': self.ρ2, 'notes': 'Ποσοστό του θλιβόμενου οπλισμού'})
        data.append({'quantity': 'ρv', 'value': self.ρv, 'notes': 'Ποσοστό του ενδιάμεσου οπλισμού'})
        data.append({'quantity': 'ρtot', 'value': self.ρtot, 'notes': 'Συνολικό ποσοστό του διαμήκους οπλισμού'})
        data.append({'quantity': 'ρd', 'value': self.ρd, 'notes': 'Ποσοστό του δισδιαγώνιου οπλισμού'})
        data.append({'quantity': 'ω1', 'value': self.ω1, 'notes': 'Μηχανικό ποσοστό του εφελκυόμενου οπλισμού'})
        data.append({'quantity': 'ω2', 'value': self.ω2, 'notes': 'Μηχανικό ποσοστό του θλιβόμενου οπλισμού'})
        data.append({'quantity': 'ωv', 'value': self.ωv, 'notes': 'Μηχανικό ποσοστό του ενδιάμεσου οπλισμού'})
        data.append({'quantity': 'ωtot', 'value': self.ωtot, 'notes': 'Συνολικό μηχανικό ποσοστό του διαμήκους οπλισμού'})
        data.append({'quantity': 'ωd', 'value': self.ωd, 'notes': 'Μηχανικό ποσοστό του δισδιαγώνιου οπλισμού'})
        data.append({'quantity': 'ρw', 'value': self.ρw, 'notes': 'Ποσοστό του εγκάρσιου οπλισμού'})
        return OutputTable(data=data)


class RectangularBeamProperties:
    def __init__(self, b, h):
        self.b = b
        self.h = h

    def area(self):
        return self.b * self.h

    def moment_of_inertia_yy(self):
        return self.b ** 3 * self.h / 12

    def moment_of_inertia_xx(self):
        return self.h ** 3 * self.b / 12

    def torsional_constant(self):
        return self.b ** 3 * self.h * (1 / 3 - 0.21 * self.b / self.h * (1 - self.b ** 4 / (12 * self.h ** 4)))

    def shear_area_2(self):
        return 5 / 6 * self.area()

    def shear_area_3(self):
        return 5 / 6 * self.area()

    def section_modulus_2(self):
        return self.b ** 2 * self.h / 6

    def section_modulus_3(self):
        return self.h ** 2 * self.b / 6

    def plastic_modulus_2(self):
        return self.b ** 2 * self.h / 4

    def plastic_modulus_3(self):
        return self.h ** 2 * self.b / 4

    def radius_of_gyration_2(self):
        return self.b / 12 ** 0.5

    def radius_of_gyration_3(self):
        return self.h / 12 ** 0.5

    def x_g(self):
        return self.b / 2

    def y_g(self):
        return self.h / 2

    def __str__(self):
        _str = 'Rectangular beam geometric properties\n'
        _str += f'b={self.b:.3f} [m]\n'
        _str += f'h={self.h:.3f} [m]\n'
        _str += f'A={self.area():.3f} [m2]\n'
        _str += f'Iyy={self.moment_of_inertia_yy():.3E} [m4]\n'
        _str += f'Ixx={self.moment_of_inertia_xx():.3E} [m4]\n'
        _str += f'J={self.torsional_constant():.3E} [m4]\n'
        _str += f'ShearArea2={self.shear_area_2():.4f} [m2]\n'
        _str += f'ShearArea3={self.shear_area_3():.4f} [m2]\n'
        _str += f'SectionModulus2={self.section_modulus_2():.4f} [m3]\n'
        _str += f'SectionModulus3={self.section_modulus_3():.4f} [m3]\n'
        _str += f'PlasticModulus2={self.plastic_modulus_2():.4f} [m3]\n'
        _str += f'PlasticModulus3={self.plastic_modulus_3():.4f} [m3]\n'
        _str += f'RadiusOfGyration2={self.radius_of_gyration_2():.4f} [m]\n'
        _str += f'RadiusOfGyration3={self.radius_of_gyration_3():.4f} [m]\n'
        _str += f'xG={self.x_g():.3f} [m]\n'
        _str += f'yG={self.y_g():.3f} [m]\n'
        return _str



import pandas as pd
import numpy as np


def frame_element_results(results_dataframe, element, quantity, location):
    _df = results_dataframe.loc[results_dataframe['Frame'] == element]
    return _df[quantity].iloc[location]



import numpy as np
import matplotlib.pyplot as plt
from dataclasses import dataclass


@dataclass
class KapposConfined91:
    fc: float
    εco: float
    ρw: float
    bc: float
    s: float
    fyw: float
    hoops_pattern: int  # (1=single,2=double,3=multi)

    def __post_init__(self):
        if self.hoops_pattern == 1:
            α = 0.55
            b = 0.75
        elif self.hoops_pattern == 2:
            α = 1.0
            b = 1.0
        elif self.hoops_pattern == 3:
            α = 1.25
            b = 1.0
        else:
            print('Check hoops pattern!')

        ωw = self.ρw * self.fyw / self.fc
        k = 1 + α * ωw**b

        self.fcc = k * self.fc
        self.εcco = k * k * self.εco
        self.u = 0.50 / ((0.75 * self.ρw * (self.bc / self.s)**0.5) +
                         (3 + 0.29 * (self.fc / k)) / (145 * (self.fc / k) - 1000.) - self.εco)


    def σc(self, εc):
        if εc <= self.εcco:
            return self.fcc * (2 * (εc / self.εcco) - (εc / self.εcco)**2)
        if εc > self.εcco:
            return max(self.fcc * (1 - self.u * (εc - self.εcco)), 0.25 * self.fcc)



def os_to_sap_forces_col(col_results):
    # Στύλοι: -V1 -N1 M1 V2 N2 -M2
    return[-col_results[1], -col_results[0], col_results[2], col_results[4], col_results[3], -col_results[5]]

def os_to_sap_forces_beam(col_results):
    # Δοκοί: (±)N1 -V1 -M1 (±)N2 V2 M2
    # Για το αξονικό δεν το έλεγξα γιατί δεν υπήρχε στα παραδείγματα λόγω διαφράγματος
    return[col_results[0], -col_results[1], -col_results[2], col_results[3], col_results[4], col_results[5]]